@@ -36,6 +36,7 @@ protected:
    virtual void execListCheck() = 0;
    virtual void setTableSetOpt() = 0;
    virtual void execTableInfo() = 0;
+   virtual void execShowUptime() = 0;
    virtual void execShowPool() = 0;
    virtual void execShowSystemSpace() = 0;
    virtual void execTableDesc() = 0;
@@ -55,8 +56,10 @@ protected:
    virtual void execEnableAutoCommit() = 0;
    virtual void execDisableAutoCommit() = 0;
    virtual void execSetIsolation() = 0;
-   virtual void execEnableQueryCache() = 0;
-   virtual void execDisableQueryCache() = 0;
+   virtual void execEnableUpdateSync() = 0;
+   virtual void execDisableUpdateSync() = 0;
+   virtual void execEnableBTreeCache() = 0;
+   virtual void execDisableBTreeCache() = 0;
    virtual void execTableReorganize() = 0;
    virtual void execIndexReorganize() = 0;
    virtual void execSelect() = 0;
@@ -107,6 +110,7 @@ protected:
    virtual void procNoopStatement() = 0;
    virtual void procInitialAssignment1() = 0;
    virtual void procInitialAssignment2() = 0;
+   virtual void procThrowStatement() = 0;
    virtual void procQueryStatement() = 0;
    virtual void procReturnStatement1() = 0;
    virtual void procReturnStatement2() = 0;
@@ -220,9 +224,11 @@ protected:
    virtual void alterDropCol() = 0;
    virtual void alterAddCol() = 0;
    virtual void alterModCol() = 0;
+   virtual void alterModDefault() = 0;
    virtual void alterRenameCol() = 0;
    virtual void renameTable() = 0;
    virtual void renameIndex() = 0;
+   virtual void renameBTree() = 0;
    virtual void renameKey() = 0;
    virtual void renameProcedure() = 0;
    virtual void renameView() = 0;
@@ -263,6 +269,7 @@ protected:
    virtual void miscDecimalType() = 0;
    virtual void miscFixedType() = 0;
    virtual void miscBlobType() = 0;
+   virtual void miscClobType() = 0;
    virtual void miscEqualComp() = 0;
    virtual void miscNotEqualComp() = 0;
    virtual void miscLessComp() = 0;
@@ -292,6 +299,7 @@ protected:
    virtual void miscTrueValue() = 0;
    virtual void miscFalseValue() = 0;
    virtual void miscBlobRef() = 0;
+   virtual void miscClobRef() = 0;
    virtual void miscDateFormat() = 0;
    virtual void miscDateValue() = 0;
    virtual void setObject1() = 0;
@@ -328,6 +336,7 @@ protected:
       VALUES,
       VAR,
       UPPER,
+      UPTIME,
       UPDATE,
       UNIQUE,
       UNION,
@@ -341,6 +350,7 @@ protected:
       TRANSACTION,
       TMPOBJ,
       TINYINT,
+      THROW,
       THEN,
       TABLESET,
       TABLE,
@@ -373,7 +383,6 @@ protected:
       RANDINT,
       RANDSTR,
       QUIT,
-      QUERYCACHE,
       PROCEDURE,
       PRINT,
       PRIMARY,
@@ -413,8 +422,8 @@ protected:
       INTO,
       INT,
       INNER,
-      INDEX,
       INTOK,
+      INDEX,
       IF,
       IDENTIFIED,
       HAVING,
@@ -452,12 +461,17 @@ protected:
       COLUMN,
       COMMIT,
       CLOSE,
+      CLOB2STR,
+      CLOBSIZE,
+      CLOB,
       CHECK,
       CASE,
       CALL,
+      CACHE,
       BY,
       BTREE,
       BOOL,
+      BLOBSIZE,
       BLOB,
       BITXOR,
       BITOR,
@@ -465,6 +479,7 @@ protected:
       BIGINT,
       BETWEEN,
       BEGIN,
+      AVLTREE,
       AVG,
       AUTOCOMMIT,
       AUTHORIZE,
@@ -539,6 +554,7 @@ private:
       SimpleAttr,
       IndexOption,
       IndexType,
+      BtreeType,
       StartStatement,
       CommitStatement,
       RollbackStatement,
@@ -847,6 +863,25 @@ private:
    void loadParser240();
    void loadParser241();
    void loadParser242();
+   void loadParser243();
+   void loadParser244();
+   void loadParser245();
+   void loadParser246();
+   void loadParser247();
+   void loadParser248();
+   void loadParser249();
+   void loadParser250();
+   void loadParser251();
+   void loadParser252();
+   void loadParser253();
+   void loadParser254();
+   void loadParser255();
+   void loadParser256();
+   void loadParser257();
+   void loadParser258();
+   void loadParser259();
+   void loadParser260();
+   void loadParser261();
 
    class ScannerStateEntry {
 
@@ -943,8 +978,8 @@ class ScannerTransEntry {
       int _num;
    };
 
-   ActionEntry _actionMap[831][178];
-   int _jumpMap[831][117];
+   ActionEntry _actionMap[868][185];
+   int _jumpMap[868][118];
 
    class ProdEntry {
 
