@@ -6,22 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //         
-// (C)opyright 2000-2013 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -29,7 +14,7 @@
 // 
 // Description: 
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -50,10 +35,13 @@ CegoAVLIndexManager::CegoAVLIndexManager(CegoTableManager *pTM)
     _pLogger = pTM->getDBMng();
     _modId = pTM->getDBMng()->getModId("CegoAVLIndexManager");
     _pTM = pTM;
+    _lockId = 0;
 }
 
 CegoAVLIndexManager::~CegoAVLIndexManager()
 {
+    if (_lockId)
+	_pTM->getLockHandler()->unlockData(CegoObject::BTREE, _lockId);
 }
 
 void CegoAVLIndexManager::insertNativeIndexTable(CegoTableObject& ioe, 
@@ -61,7 +49,7 @@ void CegoAVLIndexManager::insertNativeIndexTable(CegoTableObject& ioe,
 					      const CegoDataPointer& dp,
 					      char* idxPtr, 
 					      int idxLen, 
-					      int tid, 
+					      unsigned long long tid, 
 					      bool insertAtLastPage,
 					      CegoDataPointer& ritp)
 {
@@ -89,13 +77,16 @@ void CegoAVLIndexManager::insertNativeIndexTable(CegoTableObject& ioe,
     p = (char*)pC->getFirst(len, rdp);
     
     // root datapointer must not be zero
-    if ( p == 0)
+    if ( p == 0 )
     {
 	pC->abort();
 	delete pC;	
 	throw Exception(EXLOC, "Missing Index Anchor");
     }
 
+    // cout << "Write Locking " << rdp.getFileId() << " " << rdp.getPageId()  << endl;
+    _lockId = _pTM->getLockHandler()->lockData(CegoObject::BTREE, rdp.getFileId(), rdp.getPageId(), CegoLockHandler::WRITE);
+    
     CegoAVLIndexEntry base;
     base.setPtr(p, len);
 
@@ -104,7 +95,7 @@ void CegoAVLIndexManager::insertNativeIndexTable(CegoTableObject& ioe,
     // is this the first entry ?
     if ( base.getRightBranch() == nil )
     {	
-	CegoAVLIndexEntry nie(tid); 
+	CegoAVLIndexEntry nie; 
 	
 	nie.initEntry(dp, idxPtr, idxLen);
 	nie.setParent(rdp);
@@ -149,12 +140,18 @@ void CegoAVLIndexManager::insertNativeIndexTable(CegoTableObject& ioe,
 
 	bool isUnique = false;
 	
-	if ( idxType == CegoObject::UINDEX || idxType == CegoObject::PINDEX )
+	if ( idxType == CegoObject::UAVLTREE || idxType == CegoObject::PAVLTREE )
 	    isUnique = true;
 	
 	insertIndexTable(ioe, sysEntry, ritp, isUnique, dp, idxPtr, idxLen, tid, insertAtLastPage);
 
     }
+
+    if ( _lockId )
+    {
+	_pTM->getLockHandler()->unlockData(CegoObject::BTREE, _lockId);
+	_lockId = 0;
+    }
 }
 
 void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
@@ -164,11 +161,17 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 					const CegoDataPointer& dp, 
 					char* idxPtr, 
 					int idxLen, 
-					int tid,
+					unsigned long long tid,
 					bool insertAtLastPage, 
 					bool allowWrite)
 {
 
+
+
+    // cout << "Write Locking " << ritp.getFileId() << " " << ritp.getPageId()  << endl;
+    if ( _lockId == 0 )
+	_lockId = _pTM->getLockHandler()->lockData(CegoObject::BTREE, ritp.getFileId(), ritp.getPageId(), CegoLockHandler::WRITE);
+          
     int tabSetId = ioe.getTabSetId();
     ListT<CegoField> schema = ioe.getSchema();
  
@@ -182,7 +185,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
     else
 	fixMode = CegoBufferPool::SYNC;
 
-    unsigned long lockId = _pTM->claimDataPtr(tabSetId, CegoLockHandler::WRITE, fixMode, ritp, p, len);
+    CegoBufferPage rootPage = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ritp, p, len);
 
     CegoAVLIndexEntry base;
     base.setPtr(p, len);
@@ -238,23 +241,18 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 		// the key may be duplictate if an update operation occurs ( forced transaction )
 		// in this case we allow double entries ( cleanup by commit )
 
-		unsigned long dataLock;
-		dataLock = _pTM->claimDataPtr(tabSetId, CegoLockHandler::READ, fixMode, ie.getData(), p, len);
+		CegoBufferPage dataPage = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ie.getData(), p, len);
 		
 		// skipping tid
-		int dataTid = *(int*)p;
-		int tastep;
+		unsigned long long dataTid; //  = *(unsigned long long*)p;
+		unsigned long long tastep;
 		CegoTupleState ts;
 
-		char* ptastep = p + sizeof(int);
-		memcpy(&tastep, ptastep, sizeof(int));
-		
-		char* pts = p + sizeof(int) + sizeof(int);
-		memcpy(&ts, pts, sizeof(CegoTupleState));
+		CegoQueryHelper::decodeTupleHeader(dataTid, tastep, ts, p);
 
-		_pTM->releaseDataPtr(dataLock, false);
+		_pTM->releaseDataPtrUnlocked(dataPage, false);
 	    
-		if (tid == dataTid && ( ts == DELETED || ts == OBSOLETE ))
+		if ( tid == dataTid && ( ts == DELETED || ts == OBSOLETE ))
 		{
 		    leftBranch = true;
 		    ptp = itp;
@@ -262,9 +260,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 		}
 		else
 		{
-		    if (lockId)
-			_pTM->releaseDataPtr(lockId, false);
-		    
+		    _pTM->releaseDataPtrUnlocked(rootPage, false);		    
 		    _pTM->releaseDataPtrUnlocked(pageIE, false);
 
 		    throw Exception(EXLOC, Chain("Duplicate index key on unique index ") + ioe.getName() );
@@ -273,8 +269,8 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 	    }	  
 	    else
 	    {
-		if (lockId)
-		    _pTM->releaseDataPtr(lockId, false);
+		
+		_pTM->releaseDataPtrUnlocked(rootPage, false);
 		_pTM->releaseDataPtrUnlocked(pageIE, false);
 		
 		throw Exception(EXLOC, Chain("Duplicate index key on unique index ") + ioe.getName() );
@@ -306,12 +302,9 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 		
 	    }
 	    catch ( Exception e )
-	    {
-		
+	    {		
 		_pTM->releaseDataPtrUnlocked(pageIE, false);
-		if (lockId)
-		    _pTM->releaseDataPtr(lockId, false);
-
+		_pTM->releaseDataPtrUnlocked(rootPage, false);
 		throw e;
 	    }
 	    
@@ -356,13 +349,20 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 	}
     }
     
-     if (level ==  1 )
+     if ( level ==  1 )
      {
 	 // no balance on tree level 1
-	if (lockId)
-	    _pTM->releaseDataPtr(lockId, true);
+	 _pTM->releaseDataPtrUnlocked(rootPage, true);
+	 
 	if (pageIE.isFixed())
 	    _pTM->releaseDataPtrUnlocked(pageIE, true);
+
+
+	if ( _lockId )
+	{
+	    _pTM->getLockHandler()->unlockData(CegoObject::BTREE, _lockId);
+	    _lockId = 0;
+	}
 	
 	return;
      }
@@ -378,12 +378,12 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
     CegoAVLIndexEntry pe;       
     pe.setPtr(p, len);
 
-    while ( treatBalance && level > 1)
+    while ( treatBalance && level > 1 )
     {
 	char leftHeight, rightHeight;
 	getSubTreeHeight(tabSetId, fixMode, pe, leftHeight, rightHeight);
 
-	if (pe.getRightBranch() == itp)	    
+	if ( pe.getRightBranch() == itp )	    
 	{			    
 	    if ( rightHeight - leftHeight == 1 ) 
 	    {		
@@ -400,13 +400,13 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 
 		ptp = ie.getParent();
 		
-		if (ptp.getOffset())
+		if ( ptp.getOffset() )
 		{   
 		    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 		    pe.setPtr(p, len);
 		}
 	    }
-	    else if (leftHeight == rightHeight)
+	    else if ( leftHeight == rightHeight )
 	    {
 		if ( leftHeight != pe.getHeight() )
 		{
@@ -422,7 +422,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 		char subLeftHeight, subRightHeight;
 		getSubTreeHeight(tabSetId, fixMode, ie, subLeftHeight, subRightHeight);
 
-		if (subLeftHeight < subRightHeight)
+		if ( subLeftHeight < subRightHeight )
 		{		    
 		    rotateRR(tabSetId, ptp, fixMode);		       
 		}
@@ -450,7 +450,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 
 		ptp = ie.getParent();
 
-		if (ptp.getOffset())
+		if ( ptp.getOffset() )
 		{
 		    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 		    pe.setPtr(p, len);    
@@ -472,7 +472,7 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 		char subLeftHeight, subRightHeight;
 		getSubTreeHeight(tabSetId, fixMode, ie, subLeftHeight, subRightHeight);
 	
-		if (subLeftHeight > subRightHeight)
+		if ( subLeftHeight > subRightHeight )
 		{		    
 		    rotateLL(tabSetId, ptp, fixMode);		  
 		}
@@ -490,9 +490,14 @@ void CegoAVLIndexManager::insertIndexTable(CegoTableObject& ioe,
 	_pTM->releaseDataPtrUnlocked(pagePE, true);
     if ( pageIE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageIE, true);
-    if (lockId)
-	_pTM->releaseDataPtr(lockId, true);
+    if ( rootPage.isFixed() )
+	_pTM->releaseDataPtrUnlocked(rootPage, true);
 
+    if ( _lockId )
+    {
+	_pTM->getLockHandler()->unlockData(CegoObject::BTREE, _lockId);
+	_lockId = 0;
+    }
 }
 
 void CegoAVLIndexManager::getSubTreeHeight(int tabSetId, CegoBufferPool::FixMode fixMode, const CegoAVLIndexEntry& pe, char& leftHeight, char& rightHeight)
@@ -560,7 +565,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
     int len;
     p = (char*)pC->getFirst(len, rdp);
     
-    if ( p == 0)
+    if ( p == 0 )
     {
 	throw Exception(EXLOC, "Missing index anchor for index <" + indexName  + ">");
     }
@@ -571,12 +576,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
     /////////////////////////////////////////
 
     CegoDataPointer ptp = rdp;
-    unsigned long rootLock = _pTM->claimDataPtr(tabSetId, CegoLockHandler::WRITE, fixMode,  ptp, p, len);
-
-    // CegoTableObject ioe, toe;
-    
-    // getObject(tabSetId, tableName, tabType, toe);
-    // getObject(tabSetId, indexName, idxType, ioe);
+    CegoBufferPage rootPage = _pTM->claimDataPtrUnlocked(tabSetId, fixMode,  ptp, p, len);
 
     CegoAVLIndexEntry ie;
     ie.setPtr(p, len);    
@@ -669,7 +669,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		CegoBufferPage pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 		pie.setPtr(p, len);	  	
 		
-		if (pie.getLeftBranch() == itp)
+		if ( pie.getLeftBranch() == itp )
 		{
 		    // cout << "Index Delete : case Ia " << endl;
 		    
@@ -711,7 +711,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			}
 		    }
 		}
-		else if ( pie.getRightBranch() == itp)
+		else if ( pie.getRightBranch() == itp )
 		{
 		    
 		    // cout << "Index Delete : case Ib " << endl;
@@ -977,7 +977,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 		CegoBufferPage pageNE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ndp, p, len);
 		nie.setPtr(p, len);
 		
-		if ( nie.getLeftBranch() == nil)
+		if ( nie.getLeftBranch() == nil )
 		{
 		    
 		    // cout << "Index Delete : case IVa " << endl;
@@ -1160,21 +1160,21 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 			_pTM->releaseDataPtrUnlocked(pageRE, true);
 		    }
 		    
-		    if (pageAE.isFixed())
+		    if ( pageAE.isFixed() )
 			_pTM->releaseDataPtrUnlocked(pageAE, true);	
-		    if (pageBE.isFixed())
+		    if ( pageBE.isFixed() )
 			_pTM->releaseDataPtrUnlocked(pageBE, true);	
-		    if (pageCE.isFixed())
+		    if ( pageCE.isFixed() )
 			_pTM->releaseDataPtrUnlocked(pageCE, true);	
-		    if (pageFE.isFixed())
+		    if ( pageFE.isFixed() )
 			_pTM->releaseDataPtrUnlocked(pageFE, true);	
-		    if (pageZE.isFixed())
+		    if ( pageZE.isFixed() )
 			_pTM->releaseDataPtrUnlocked(pageZE, true);		       
 		    
 		    _pTM->deleteData(idxType, tabSetId, itp);
 		}
 		
-		if (pageNE.isFixed())
+		if ( pageNE.isFixed() )
 		    _pTM->releaseDataPtrUnlocked(pageNE, true);
 	    }
 	}
@@ -1187,7 +1187,7 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
 
 	    Chain v;
 
-	    while (pF)
+	    while ( pF )
 	    {
 				
 		int flen;
@@ -1213,17 +1213,15 @@ void CegoAVLIndexManager::deleteIndexTable(int tabSetId, const Chain& tableName,
     }
     catch ( Exception e )
     {
-	
-	if (rootLock)
-	    _pTM->releaseDataPtr(rootLock, false);
+	_pTM->releaseDataPtrUnlocked(rootPage, false);
 	throw e;
     }
     
-    if (pageIE.isFixed())
+    if ( pageIE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageIE, false);
     
-    if (rootLock)
-	_pTM->releaseDataPtr(rootLock, false);
+   
+    _pTM->releaseDataPtrUnlocked(rootPage, false);
     
 }
 
@@ -1294,7 +1292,7 @@ void CegoAVLIndexManager::propagateDecrease(int tabSetId, CegoDataPointer itp, C
 	    {
 		_pTM->releaseDataPtrUnlocked(pagePE, true);
 	    }
-	    if (ptp.getOffset())
+	    if ( ptp.getOffset() )
 	    {   
 		pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 		pie.setPtr(p, len);
@@ -1312,7 +1310,7 @@ void CegoAVLIndexManager::propagateDecrease(int tabSetId, CegoDataPointer itp, C
 	
 	ptp = ie.getParent();
 	
-	if (ptp.getOffset())
+	if ( ptp.getOffset() )
 	{   
 	    pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 	    pie.setPtr(p, len);
@@ -1427,7 +1425,7 @@ CegoAVLIndexManager::TupleOrder CegoAVLIndexManager::compIndexValue(const ListT<
 
     CegoField* pF = schema.First();
            	
-    while (pF)
+    while ( pF )
     {
        
 	int flen1, flen2;
@@ -1505,7 +1503,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
 
     atp = pe.getParent();
           
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
 	ae.setPtr(p, len);
@@ -1513,7 +1511,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
 
     itp = pe.getRightBranch();
     
-    if (itp.getOffset())
+    if ( itp.getOffset() )
     {
 	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
 	ie.setPtr(p, len);
@@ -1533,12 +1531,12 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
     /////////////////////
     
     pe.setRightBranch(ctp);
-    if (ctp.getOffset())
+    if ( ctp.getOffset() )
     {
 	ce.setParent(ptp);			
     }
     
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {			
 	if ( ae.getRightBranch() == ptp )
 	{
@@ -1563,19 +1561,19 @@ CegoDataPointer CegoAVLIndexManager::rotateRR(int tabSetId, const CegoDataPointe
     getSubTreeHeight(tabSetId, fixMode, ie, leftHeight, rightHeight);				   
     ie.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
 
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	getSubTreeHeight(tabSetId, fixMode, ae, leftHeight, rightHeight);
 	ae.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
     }
 
-    if (pagePE.isFixed())
+    if ( pagePE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pagePE, true);	
-    if (pageIE.isFixed())
+    if ( pageIE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageIE, true);	
-    if (pageCE.isFixed())
+    if ( pageCE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageCE, true);	
-    if (pageAE.isFixed())
+    if ( pageAE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageAE, true);	
 
     return itp;
@@ -1605,7 +1603,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
     // e is left child of c
     // f is right child of c
 
-    if (ptp.getOffset())
+    if ( ptp.getOffset() )
     {   
 	pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 	pe.setPtr(p, len);
@@ -1617,7 +1615,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
 
     atp = pe.getParent();
         
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
 	ae.setPtr(p, len);
@@ -1625,9 +1623,8 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
 
     itp = pe.getRightBranch();    
 
-    if (itp.getOffset())
+    if ( itp.getOffset() )
     {
-	
 	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
 	ie.setPtr(p, len);
     }
@@ -1644,7 +1641,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
     }
     
     ftp = ce.getRightBranch();
-    if (ftp.getOffset())
+    if ( ftp.getOffset() )
     {
 	pageFE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ftp, p, len);
 	fe.setPtr(p, len);
@@ -1663,7 +1660,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
 	fe.setParent(itp);
     }
     
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	if (ae.getRightBranch() == ptp )
 	{
@@ -1680,7 +1677,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
     ie.setParent(ctp);
     
     ie.setLeftBranch(ftp);
-    if (ftp.getOffset())
+    if ( ftp.getOffset() )
     {
 	fe.setParent(itp);
     }
@@ -1688,7 +1685,7 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
     
     pe.setParent(ctp);
     
-    if (etp.getOffset())
+    if ( etp.getOffset() )
     {
 	ee.setParent(ptp);
     }		    
@@ -1706,23 +1703,23 @@ CegoDataPointer CegoAVLIndexManager::rotateRL(int tabSetId, const CegoDataPointe
     getSubTreeHeight(tabSetId, fixMode, ce, leftHeight, rightHeight);
     ce.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
 
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	getSubTreeHeight(tabSetId, fixMode, ae, leftHeight, rightHeight);
 	ae.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
     }
     
-    if (pagePE.isFixed())
+    if ( pagePE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pagePE, true);	
-    if (pageIE.isFixed())
+    if ( pageIE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageIE, true);
-    if (pageCE.isFixed())
+    if ( pageCE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageCE, true);	
-    if (pageAE.isFixed())
+    if ( pageAE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageAE, true);	
-    if (pageEE.isFixed())
+    if ( pageEE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageEE, true);	
-    if (pageFE.isFixed())
+    if ( pageFE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageFE, true);	
 
     return ctp;
@@ -1749,7 +1746,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
     // a is parent of pe
     // d is right child of ie
 
-    if (ptp.getOffset())
+    if ( ptp.getOffset() )
     {   
 	pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 	pe.setPtr(p, len);
@@ -1761,7 +1758,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
     
     atp = pe.getParent();
     
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {	
 	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
 	ae.setPtr(p, len);
@@ -1769,7 +1766,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
 
     itp = pe.getLeftBranch();
 
-    if (itp.getOffset())
+    if ( itp.getOffset() )
     {
 	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
 	ie.setPtr(p, len);
@@ -1781,13 +1778,13 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
     
     dtp = ie.getRightBranch();
     
-    if (dtp.getOffset())
+    if ( dtp.getOffset() )
     {
 	pageDE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, dtp, p, len);
 	de.setPtr(p, len);
     }
         
-    if (dtp.getOffset())
+    if ( dtp.getOffset() )
     {
 	de.setParent(ptp);
     }
@@ -1796,7 +1793,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
     pe.setParent(itp);
     ie.setRightBranch(ptp);		
     
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	
 	if ( ae.getRightBranch() == ptp )
@@ -1817,19 +1814,19 @@ CegoDataPointer CegoAVLIndexManager::rotateLL(int tabSetId, const CegoDataPointe
     getSubTreeHeight(tabSetId, fixMode, ie, leftHeight, rightHeight);				   
     ie.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
 
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	getSubTreeHeight(tabSetId, fixMode, ae, leftHeight, rightHeight);				   
 	ae.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
     }
 
-    if (pagePE.isFixed())
+    if ( pagePE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pagePE, true);	    
-    if (pageIE.isFixed())
+    if ( pageIE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageIE, true);	
-    if (pageAE.isFixed())
+    if ( pageAE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageAE, true);	
-    if (pageDE.isFixed())
+    if ( pageDE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageDE, true);	
 
     return itp;
@@ -1860,7 +1857,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     // c is left child of d
     // e is right child of d
     
-    if (ptp.getOffset())
+    if ( ptp.getOffset() )
     {   
 	pagePE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ptp, p, len);
 	pe.setPtr(p, len);
@@ -1872,7 +1869,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     
     atp = pe.getParent();
     
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {
 	pageAE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, atp, p, len);
 	ae.setPtr(p, len);
@@ -1880,7 +1877,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
 
     itp = pe.getLeftBranch();
     
-    if (itp.getOffset())
+    if ( itp.getOffset() )
     {
 	pageIE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, itp, p, len);
 	ie.setPtr(p, len);
@@ -1891,7 +1888,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     }
     
     dtp = ie.getRightBranch();
-    if (dtp.getOffset())
+    if ( dtp.getOffset() )
     {	
 	pageDE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, dtp, p, len);
 	de.setPtr(p, len);
@@ -1902,14 +1899,14 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     }
 
     ctp = de.getLeftBranch();
-    if (ctp.getOffset())
+    if ( ctp.getOffset() )
     {
 	pageCE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, ctp, p, len);
 	ce.setPtr(p, len);
     }
 
     etp = de.getRightBranch();
-    if (etp.getOffset())
+    if ( etp.getOffset() )
     {
 	pageEE = _pTM->claimDataPtrUnlocked(tabSetId, fixMode, etp, p, len);
 	ee.setPtr(p, len);
@@ -1917,7 +1914,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     
     ie.setRightBranch(ctp);
     
-    if (ctp.getOffset())
+    if ( ctp.getOffset() )
     {
 	ce.setParent(itp);
     }
@@ -1925,7 +1922,7 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     ie.setParent(dtp);
     de.setLeftBranch(itp);		    
     
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {		    
 	if ( ae.getRightBranch() == ptp )		    
 	{
@@ -1941,13 +1938,12 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     de.setRightBranch(ptp);			
     pe.setParent(dtp);		
     
-    if (etp.getOffset())
+    if ( etp.getOffset() )
     {
 	ee.setParent(ptp);
     }
     pe.setLeftBranch(etp);
     
-
     // set up balance
 
     char leftHeight, rightHeight;
@@ -1960,24 +1956,24 @@ CegoDataPointer CegoAVLIndexManager::rotateLR(int tabSetId, const CegoDataPointe
     getSubTreeHeight(tabSetId, fixMode, de, leftHeight, rightHeight); 
     de.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
 
-    if (atp.getOffset())
+    if ( atp.getOffset() )
     {		    
 	getSubTreeHeight(tabSetId, fixMode, ae, leftHeight, rightHeight); 
 	ae.setHeight(leftHeight > rightHeight ? leftHeight + 1 : rightHeight + 1);
     }
 
-    if (pagePE.isFixed())
+    if ( pagePE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pagePE, true);	
-    if (pageIE.isFixed())
+    if ( pageIE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageIE, true);	
     
-    if (pageAE.isFixed())
+    if ( pageAE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageAE, true);	
-    if (pageCE.isFixed())
+    if ( pageCE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageCE, true);	
-    if (pageDE.isFixed())
+    if ( pageDE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageDE, true);	
-    if (pageEE.isFixed())
+    if ( pageEE.isFixed() )
 	_pTM->releaseDataPtrUnlocked(pageEE, true);
 
     return dtp;
@@ -1989,8 +1985,7 @@ char CegoAVLIndexManager::checkIndex(int tabSetId, const Chain& indexName, CegoO
 
     CegoTableObject oe;
     _pTM->getObject(tabSetId, indexName, idxType, oe);
-    
-
+   
     CegoObjectCursor* pC = _pTM->getObjectCursor(tabSetId, oe.getTabName(), indexName, idxType);
     
     if ( ! pC )
@@ -2004,7 +1999,7 @@ char CegoAVLIndexManager::checkIndex(int tabSetId, const Chain& indexName, CegoO
     int len;
     p = (char*)pC->getFirst(len, rdp);
     
-    if ( p == 0)
+    if ( p == 0 )
     {
 	throw Exception(EXLOC, "Missing Index Anchor");
     }
@@ -2015,7 +2010,7 @@ char CegoAVLIndexManager::checkIndex(int tabSetId, const Chain& indexName, CegoO
     /////////////////////////////////////////
 
     CegoDataPointer ptp = rdp;
-    unsigned long rootLock = _pTM->claimDataPtr(tabSetId, CegoLockHandler::READ, CegoBufferPool::SYNC, ptp, p, len);
+    CegoBufferPage rootPage = _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, ptp, p, len);
 
     CegoAVLIndexEntry ie;
     ie.setPtr(p, len);    
@@ -2025,8 +2020,7 @@ char CegoAVLIndexManager::checkIndex(int tabSetId, const Chain& indexName, CegoO
 
     char h = recursiveIndexNodeCheck(tabSetId, itp);
     
-    if (rootLock)
-	_pTM->releaseDataPtr(rootLock, false);
+    _pTM->releaseDataPtrUnlocked(rootPage, false);
 
     return h;
 }
@@ -2042,7 +2036,6 @@ char CegoAVLIndexManager::recursiveIndexNodeCheck(int tabSetId, const CegoDataPo
     char* p;
     int len;
 
-    
 #ifdef CGDEBUG
     _pLogger->log(_modId, Logger::DEBUG, Chain("Claiming datapointer ") +  dp.toChain());
 #endif
@@ -2059,7 +2052,7 @@ char CegoAVLIndexManager::recursiveIndexNodeCheck(int tabSetId, const CegoDataPo
     _pLogger->log(_modId, Logger::DEBUG, Chain("Accessing datapointer ") +  dp.toChain());
 #endif
 
-    if (bp.isFixed())
+    if ( bp.isFixed() )
     {
 #ifdef CGDEBUG
 	_pLogger->log(_modId, Logger::DEBUG, Chain("Releasing datapointer ") +  dp.toChain());
