@@ -97,11 +97,25 @@ protected:
    virtual void setCheckpointAction() = 0;
    virtual void setTSSysSizeAction() = 0;
    virtual void setTSTmpSizeAction() = 0;
+   virtual void setTSAppSizeAction() = 0;
    virtual void setTSRootPathAction() = 0;
-   virtual void setTSTicketAction() = 0;
    virtual void setTSLogNumAction() = 0;
    virtual void setTSLogSizeAction() = 0;
    virtual void setTSSortAreaSizeAction() = 0;
+   virtual void enableAutoCorrectAction() = 0;
+   virtual void disableAutoCorrectAction() = 0;
+   virtual void enableTableCacheAction() = 0;
+   virtual void disableTableCacheAction() = 0;
+   virtual void enableQueryCacheAction() = 0;
+   virtual void disableQueryCacheAction() = 0;
+   virtual void listQueryCacheAction() = 0;
+   virtual void cleanQueryCacheAction() = 0;
+   virtual void setQueryCacheEntryAction() = 0;
+   virtual void setQueryCacheSizeAction() = 0;
+   virtual void listTableCacheAction() = 0;
+   virtual void cleanTableCacheAction() = 0;
+   virtual void setTableCacheEntryAction() = 0;
+   virtual void setTableCacheSizeAction() = 0;
    virtual void exportTableAction() = 0;
    virtual void exportTableSetAction() = 0;
    virtual void exportTableSetStructureAction() = 0;
@@ -124,9 +138,9 @@ protected:
    virtual void storeTmpSizeAttr() = 0;
    virtual void storeLogSizeAttr() = 0;
    virtual void storeLogNumAttr() = 0;
+   virtual void storeAppSizeAttr() = 0;
    virtual void storeSortAreaAttr() = 0;
    virtual void storeTSRootAttr() = 0;
-   virtual void storeTSTicketAttr() = 0;
    virtual void storePit() = 0;
    virtual void getDbSpecAction() = 0;
    virtual void getDbInfoAction() = 0;
@@ -154,6 +168,7 @@ protected:
       THREAD,
       TIMEOUT,
       TABLESET,
+      TABLECACHE,
       TABLE,
       SYSSIZE,
       SYNC,
@@ -177,6 +192,7 @@ protected:
       RELOCATE,
       RAWMODE,
       QUIT,
+      QUERYCACHE,
       PRIMARY,
       POOL,
       PLAIN,
@@ -213,6 +229,7 @@ protected:
       FILTER,
       FILE,
       EXPORT,
+      ENTRY,
       END_TOKEN,
       ENABLE,
       DUMP,
@@ -227,6 +244,7 @@ protected:
       CONNECT,
       CLOSE,
       CLEANUP,
+      CLEAN,
       CHECKPOINT,
       CHECK,
       BY,
@@ -236,8 +254,10 @@ protected:
       BINARY,
       BEGIN_TOKEN,
       BACKUP,
+      AUTOCORRECT,
       ASSIGN,
       ARCHLOG,
+      APPSIZE,
       ADM,
       ADD,
       ABORT,
@@ -283,6 +303,7 @@ private:
    void loadParser11();
    void loadParser12();
    void loadParser13();
+   void loadParser14();
 
    class ScannerStateEntry {
 
@@ -379,8 +400,8 @@ class ScannerTransEntry {
       int _num;
    };
 
-   ActionEntry _actionMap[358][107];
-   int _jumpMap[358][16];
+   ActionEntry _actionMap[405][113];
+   int _jumpMap[405][16];
 
    class ProdEntry {
 
