@@ -8,38 +8,22 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
-//
-// INTERFACE MODULE
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // Class: CegoAdminHandler
 // 
-// Description: Cego admin handler
+// Description: Administration client service request handler class
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// LFC INCLUDES
 #include <lfcbase/Chain.h>
 #include <lfcbase/NetHandler.h>
-
 #include <lfcxml/XMLSuite.h>
 
+// CEGO INCLUDES
 #include "CegoTableObject.h"
 #include "CegoModule.h"
 
@@ -61,6 +45,14 @@ public:
 	LOGTHREADINFO,
 	DBSESSIONINFO,
 	COPYINFO,
+	LISTQUERYCACHE,
+	CLEANQUERYCACHE,
+	SETQUERYCACHEENTRY,
+	SETQUERYCACHESIZE,
+	LISTTABLECACHE,
+	CLEANTABLECACHE,
+	SETTABLECACHEENTRY,
+	SETTABLECACHESIZE,
 	DEFINE_TABLESET,
 	REMOVE_TABLESET,
 	RESET_TABLESET,
@@ -81,8 +73,8 @@ public:
 	SET_TSINITFILE,
 	SET_TSSYSSIZE,
 	SET_TSTMPSIZE,
+	SET_TSAPPSIZE,
 	SET_TSROOTPATH,
-	SET_TSTICKET,
 	SET_TSLOGNUM,
 	SET_TSLOGSIZE,
 	SET_TSSORTAREASIZE,
@@ -126,6 +118,11 @@ public:
 	VERIFY_TABLESET,
 	CORRECT_TABLESET,
 	DUMP_OBJECT,
+	ENABLE_TABLECACHE,	
+	DISABLE_TABLECACHE,
+	ENABLE_QUERYCACHE,	
+	DISABLE_QUERYCACHE,
+
 	/* mediator requests */
 	MED_NOTIFY,
 	MED_DEFINE_TABLESET,
@@ -156,6 +153,9 @@ public:
 	MED_CORRECT_TABLESET,
 	MED_ENABLE_ARCHLOG,	
 	MED_DISABLE_ARCHLOG,
+	MED_ENABLE_AUTOCORRECT,	
+	MED_DISABLE_AUTOCORRECT,
+
 	/* secondary requests */
 	SEC_MEDSWITCH,
 	SEC_MEDRELOCATE,
@@ -185,6 +185,7 @@ public:
     ResultType reqLockInfo(const Chain& lockCat);
     ResultType reqLockStat();
     ResultType reqPoolInfo();
+    
     ResultType reqPoolEntryList();
     ResultType reqThreadInfo();
     ResultType reqDbThreadInfo();
@@ -192,16 +193,26 @@ public:
     ResultType reqLogThreadInfo();
     ResultType reqDbSessionInfo();
     ResultType reqCopyInfo();
+
+    ResultType reqListQueryCache(const Chain& tableSet);
+    ResultType reqCleanQueryCache(const Chain& tableSet);
+    ResultType reqSetQueryCacheEntry(const Chain& tableSet, int numEntry);
+    ResultType reqSetQueryCacheSize(const Chain& tableSet, int cacheSize);
+
+    ResultType reqListTableCache(const Chain& tableSet);
+    ResultType reqCleanTableCache(const Chain& tableSet);
+    ResultType reqSetTableCacheEntry(const Chain& tableSet, int numEntry);
+    ResultType reqSetTableCacheSize(const Chain& tableSet,int cacheSize);
+    
     ResultType reqDbThreadAbort(int threadId);
     
     ResultType reqDefineTableSet(const Chain& tableSet, 
-				 const Chain& tsRoot, 
-				 const Chain& tsTicket, 
+				 const Chain& tsRoot,
 				 const Chain& primary,
 				 const Chain& secondary, 
 				 const Chain& mediator,
-				 int sysFileId, int tempFileId, int sysFileSize, int tempFileSize,
-				 int logFileSize, int logFileNum, long sortAreaSize);
+				 int sysFileId, int tempFileId, int sysFileSize, int tempFileSize, int appFileSize,
+				 int logFileSize, int logFileNum, unsigned long long sortAreaSize);
     ResultType reqRemoveTableSet(const Chain& tableSet);
     ResultType reqResetTableSet(const Chain& tableSet);
     ResultType reqCreateTableSet(const Chain& tableSet);
@@ -233,7 +244,7 @@ public:
     ResultType reqGetTransactionInfo(const Chain& tableSet);
     ResultType reqPropTSInfo(Element *pTSInfo);
     ResultType reqSetTableSetNode(const Chain& tableSet, const Chain& primary, const Chain& secondary, const Chain& mediator);
-    ResultType reqSetLSN(const Chain& tableSet, long lsn);
+    ResultType reqSetLSN(const Chain& tableSet, unsigned long long lsn);
 
     ResultType reqSetRunState(const Chain& tableSet, const Chain& runState);
     ResultType reqSetSyncState(const Chain& tableSet, const Chain& syncState);
@@ -287,13 +298,20 @@ public:
 
     ResultType reqSetTSSysSize(const Chain& tableSet, int sysSize);
     ResultType reqSetTSTmpSize(const Chain& tableSet, int tmpSize);
+    ResultType reqSetTSAppSize(const Chain& tableSet, int appSize);
     ResultType reqSetTSRootPath(const Chain& tableSet, const Chain& tsRoot);
 
-    ResultType reqSetTSTicket(const Chain& tableSet, const Chain& tsTicket);
     ResultType reqSetTSLogNum(const Chain& tableSet, int logNum);
     ResultType reqSetTSLogSize(const Chain& tableSet, int logSize);
-    ResultType reqSetTSSortAreaSize(const Chain& tableSet, int sortAreaSize);
+    ResultType reqSetTSSortAreaSize(const Chain& tableSet, unsigned long long sortAreaSize);
 
+    ResultType reqEnableTableCache(const Chain& tableSet);
+    ResultType reqDisableTableCache(const Chain& tableSet);
+    
+    ResultType reqEnableQueryCache(const Chain& tableSet);
+    ResultType reqDisableQueryCache(const Chain& tableSet);
+
+    
     ResultType sendReq(const Chain& reqType, Element* pRoot);
 
     // mediator services 
@@ -304,13 +322,12 @@ public:
     ResultType medStopTableSet(const Chain& tableSet);
 
     ResultType medDefineTableSet(const Chain& tableSet, 
-				 const Chain& tsRoot, 
-				 const Chain& tsTicket, 
+				 const Chain& tsRoot,
 				 const Chain& primary,
 				 const Chain& secondary,
-				 int sysFileSize, int tmpFileSize,
+				 int sysFileSize, int tmpFileSize, int appFileSize,
 				 int logFileSize, int logFileNum,
-				 long sortAreaSize);
+				 unsigned long long sortAreaSize);
     
     ResultType medDropTableSet(const Chain& tableSet);
     ResultType medRemoveTableSet(const Chain& tableSet);
@@ -331,7 +348,7 @@ public:
 
     ResultType medBeginBackup(const Chain& tableSet, const Chain& msg);
     ResultType medEndBackup(const Chain& tableSet, const Chain& msg, bool keepTicket);
-    ResultType medRecover(const Chain& tableSet, int pit);
+    ResultType medRecover(const Chain& tableSet, const Chain& pit);
     ResultType medSecSwitch(const Chain& tableSet);
 
     ResultType medSecRelocate(const Chain& tableSet, const Chain& hostName);
@@ -344,11 +361,15 @@ public:
     ResultType medEnableArchLog(const Chain& tableSet);
     ResultType medDisableArchLog(const Chain& tableSet);
 
+    ResultType medEnableAutoCorrect(const Chain& tableSet);
+    ResultType medDisableAutoCorrect(const Chain& tableSet);
+
     ResultType secMedSwitch(const Chain& tableSet);
     ResultType secMedRelocate(const Chain& tableSet, const Chain& hostName);
 
     RequestType acceptRequest();
 
+    void sendInfo(const Chain& info);
     void sendResponse(const Chain& msg, Element* pSubElement = 0);
     void sendError(const Chain& msg);
     void sendData(const Chain& data);
@@ -378,6 +399,8 @@ public:
     void getLogThreadInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
     void getDbSessionInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info);
     void getCopyInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
+    bool getCacheInfo(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
+    bool getCacheList(CegoTableObject& oe, ListT<ListT< CegoFieldValue > > & info, Chain& format);
 
     Element* getTableSetInfo();
 
@@ -421,15 +444,12 @@ public:
     void getTargetHost(Chain& targetHost);
     void getEscapeCmd(Chain& escCmd);
     void getTSInitFile(Chain& initFile);
-
-    void getTSSysSize(int& sysSize);
-    void getTSTmpSize(int& tmpSize);
     void getTSRootPath(Chain& tsRoot);
 
     void getTSTicket(Chain& tsRoot);
     void getTSLogNum(int& logNum);
     void getTSLogSize(int& logSize);
-    void getTSSortAreaSize(int& sortAreaSize);
+    void getTSSortAreaSize(unsigned long long& sortAreaSize);
 
     void getTimeoutValue(int& timeout);
     void getUser(Chain& user);
@@ -452,9 +472,10 @@ public:
     void getTmpFileId(int& tmpFid);
     void getSysSize(int& sysSize);
     void getTmpSize(int& tmpSize);
+    void getAppSize(int& appSize);
     void getLogFileSize(int& logFileSize);
     void getLogFileNum(int& logFileNum);
-    void getSortAreaSize(long& sortAreaSize);
+    void getSortAreaSize(unsigned long long& sortAreaSize);
     void getArchId(Chain& archId);
     void getArchPath(Chain& archPath);
     void getHostRole(Chain& hostRole);
@@ -463,10 +484,10 @@ public:
     void getDataPort(int& dataPort);
     void getAdminPort(int& adminPort);
     void getDataId(Chain& dataId);
-    void getLSN(long& lsn);
+    void getLSN(unsigned long long& lsn);
     void getRunState(Chain& runState);
     void getSyncState(Chain& syncState);
-    void getPit(int& pit);
+    void getPit(Chain& pit);
     void getIsStructure(bool& isStructure);
     void getDoLogging(bool& doLogging);
     void getTableSyncStateList(ListT<Chain>& tsList, ListT<Chain>& runList, ListT<Chain>& syncList);
@@ -477,6 +498,9 @@ public:
 
     void getObjName(Chain& objName);
     void getObjType(CegoObject::ObjectType& type);
+
+    void getCacheEntry(int& numEntry);
+    void getCacheSize(int& cacheSize);
     
 
 private:
