@@ -6,38 +6,25 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2010 Bjoern Lemke       
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke       
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoAggregation
-// 
-// Description: Aggregation container class to store an aggregation definition 
 //
-// Status: QG-2.6
+// Description: Query aggregation utility class
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// INCLUDES
+// CEGO INCLUDES
 #include "CegoAggregation.h"
 #include "CegoExpr.h"
 #include "CegoXMLdef.h"
+#include "CegoDatabaseFormater.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
@@ -47,9 +34,9 @@ CegoAggregation::CegoAggregation()
     _pExpr = 0;
 }
 
-CegoAggregation::CegoAggregation(char* buf, CegoDistManager *pGTM)
+CegoAggregation::CegoAggregation(char* buf, CegoDistManager *pGTM, int tabSetId)
 {
-    decode(buf, pGTM);
+    decode(buf, pGTM, tabSetId);
 }
 
 CegoAggregation::CegoAggregation(Element* pAE, CegoDistManager *pGTM)
@@ -77,10 +64,10 @@ void CegoAggregation::setBlock(CegoProcBlock *pBlock)
 	_pExpr->setBlock(pBlock);
 }
 
-void CegoAggregation::setFieldListArray(ListT<CegoField> *fla, int size)
+void CegoAggregation::setFieldListArray(ListT<CegoField> **pFLA)
 {
     if ( _pExpr )
-	_pExpr->setFieldListArray(fla, size);
+	_pExpr->setFieldListArray(pFLA);
 }
 
 void CegoAggregation::setAggregationId(int aid)
@@ -173,7 +160,7 @@ void CegoAggregation::encode(char *buf)
     }
 }
 
-void CegoAggregation::decode(char *buf, CegoDistManager *pGTM)
+void CegoAggregation::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
 {
     
     if ( _pExpr )
@@ -193,9 +180,8 @@ void CegoAggregation::decode(char *buf, CegoDistManager *pGTM)
     
     if ( c == 1 )
     {
-	_pExpr = new CegoExpr(pBuf, pGTM);	
+	_pExpr = new CegoExpr(pBuf, pGTM, tabSetId);	
     }
-
 }
 
 int CegoAggregation::getEncodingLength() const
@@ -264,7 +250,39 @@ void CegoAggregation::fromElement(Element *pAggElement, CegoDistManager *pGTM)
 	_type = CegoAggregation::SUM;
     else if ( aggString == Chain(XML_COUNT_VALUE) )
 	_type = CegoAggregation::COUNT;
+}
 
+Chain CegoAggregation::getId() const
+{
+    Chain s;
+    switch (_type)
+    {
+    case CegoAggregation::MIN:
+	s =  Chain("min(") + _pExpr->getId() + Chain(")");
+	break;
+    case CegoAggregation::MAX:
+	s =  Chain("max(") + _pExpr->getId() + Chain(")");
+	break;
+    case CegoAggregation::AVG:
+	s =  Chain("avg(") + _pExpr->getId() + Chain(")");
+	break;
+    case CegoAggregation::SUM:
+	s =  Chain("sum(") + _pExpr->getId() + Chain(")");
+	break;
+    case CegoAggregation::COUNT:
+    {       
+	if ( _pExpr )
+	{ 
+	    Chain distOpt;
+	    if ( _isDistinct )
+		distOpt = Chain("d");	    
+	    s = Chain("count(") + distOpt + _pExpr->getId() + Chain(")");
+	}
+	else
+	    s = Chain("count(*)");
+    }
+    }
+    return s;
 }
 
 Chain CegoAggregation::toChain() const
@@ -297,10 +315,14 @@ Chain CegoAggregation::toChain() const
 	    s = Chain("count(*)");
     }
     }
-
     return s;
 }
 
+Chain CegoAggregation::dbFormat(CegoDatabaseFormater *pForm) const
+{
+    return pForm->formatAggregation(_type, _pExpr, _isDistinct);
+}
+
 ostream& operator << (ostream& s, const CegoAggregation& ag)
 {
     s << ag.toChain();
