@@ -8,43 +8,27 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
 // Class: CegoExpOutStream
 // 
-// Description: 
+// Description: Database XML and binary export handle
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// base includes
+// LFC INCLUDES
 #include <lfcbase/Chain.h>
 #include <lfcbase/ListT.h>
-
-// xml includes
 #include <lfcxml/Element.h>
 #include <lfcxml/XMLOutStream.h>
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoDistManager.h"
+#include "CegoAdminHandler.h"
 #include "CegoObjectCursor.h"
 #include "CegoField.h"
 
@@ -52,7 +36,7 @@ class CegoExpOutStream : public XMLOutStream {
     
 public:
     
-    CegoExpOutStream(int tabSetId, const Chain& tableName, const ListT<CegoField>& schema, CegoDistManager* pGTM);
+    CegoExpOutStream(int tabSetId, const Chain& tableName, const ListT<CegoField>& schema, CegoDistManager* pGTM, CegoAdminHandler* pAH = 0);
     ~CegoExpOutStream();
     
     Element* getFirst();
@@ -63,13 +47,11 @@ private:
     Element* getRowElement(ListT<CegoField>& schema);
     
     CegoDistManager* _pGTM;
+    CegoAdminHandler* _pAH;
     CegoObjectCursor* _pOC;
     ListT<CegoField> _schema;
+    Chain _tableName;
+    long _rowCount;
     int _tabSetId;
 };
-
 #endif
-
-
-
-
