@@ -6,57 +6,48 @@
 //                                                
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2010 Bjoern Lemke  
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoExpr
-// 
-// Description:
 //
-// Status: QG-2.6
+// Description: Query expression container class
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+
+// CEGO INCLUDES
 #include "CegoExpr.h"
-
 #include "CegoXMLdef.h"
+#include "CegoDatabaseFormater.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
-CegoExpr::CegoExpr(char* buf, CegoDistManager *pGTM)
+CegoExpr::CegoExpr(char* buf, CegoDistManager *pGTM, int tabSetId)
 {
     _pTerm=0;
     _pExpr=0;
-    decode(buf, pGTM);
+    _isAgg = false;
+    decode(buf, pGTM, tabSetId);
 }
 
 CegoExpr::CegoExpr()
 {
     _pTerm=0;
     _pExpr=0;
+    _isAgg = false;
 }
 
 CegoExpr::CegoExpr(Element* pExpElement, CegoDistManager *pGTM)
 {
     _pTerm=0;
     _pExpr=0;
+    _isAgg = false;
     fromElement(pExpElement, pGTM);
 }
 
@@ -65,6 +56,7 @@ CegoExpr::CegoExpr(CegoExpr *pExpr, CegoTerm *pTerm, ExpType expType)
     _expType = expType;
     _pExpr=pExpr;
     _pTerm=pTerm;
+    _isAgg = false;
     _alias = Chain();
 }
 
@@ -73,6 +65,7 @@ CegoExpr::CegoExpr(CegoTerm *pTerm)
     _expType = TERM;
     _pTerm=pTerm;
     _pExpr = 0;
+    _isAgg = false;
 }
 
 CegoExpr::~CegoExpr()
@@ -93,15 +86,6 @@ const Chain& CegoExpr::getAlias() const
     return _alias;
 }
 
-bool CegoExpr::getTableDesc(CegoTableDesc& td) const
-{
-    if ( _pExpr )
-	return false;
-    if ( _pTerm )
-	return _pTerm->getTableDesc(td);
-    return false;
-}
-
 bool CegoExpr::checkVar(Chain& var) const
 {
     if ( _pTerm )
@@ -120,12 +104,31 @@ CegoAttrDesc* CegoExpr::checkAttr() const
     return 0;
 }
 
-void CegoExpr::setFieldListArray(ListT<CegoField> *fla, int size)
+void CegoExpr::getFunctionList(SetT<CegoObject>& funcList)
 {
     if ( _pExpr )
-	_pExpr->setFieldListArray(fla, size);
+	_pExpr->getFunctionList(funcList);
     if ( _pTerm )
-	_pTerm->setFieldListArray(fla, size);
+	_pTerm->getFunctionList(funcList);
+}
+
+void CegoExpr::setFieldListArray(ListT<CegoField>* pFL)
+{
+    _fla[0]=pFL;
+    _fla[1]=0;
+    
+    if ( _pExpr )
+	_pExpr->setFieldListArray(_fla);
+    if ( _pTerm )
+	_pTerm->setFieldListArray(_fla);
+}
+
+void CegoExpr::setFieldListArray(ListT<CegoField> **pFLA)
+{
+    if ( _pExpr )
+	_pExpr->setFieldListArray(pFLA);
+    if ( _pTerm )
+	_pTerm->setFieldListArray(pFLA);
 }
 
 void CegoExpr::setBlock(CegoProcBlock *pBlock)
@@ -152,7 +155,6 @@ void CegoExpr::getSelectQueryList(ListT<CegoSelect*>& queryList)
     }
 }
 
-
 void CegoExpr::clearAttrCache()
 {
     if ( _pTerm )
@@ -189,6 +191,7 @@ ListT<CegoField> CegoExpr::getFieldList() const
 
 ListT<CegoAttrDesc*> CegoExpr::getAttrRefList() const
 {
+    
     ListT<CegoAttrDesc*> attrList;
     switch ( _expType )
     {
@@ -300,17 +303,25 @@ CegoExpr *CegoExpr::getExpr() const
     return _pExpr;
 }
 
-ListT<CegoAggregation*> CegoExpr::getAggregationList()
+ListT<CegoAggregation*>& CegoExpr::getAggregationList()
 {
-    switch ( _expType )
+    if ( _isAgg == false )
     {
-    case ADD:
-    case SUB:
-    case CONCAT:
-	return ( _pExpr->getAggregationList() + _pTerm->getAggregationList() );	
-    case TERM:
-	return ( _pTerm->getAggregationList() );
+	switch ( _expType )
+	{
+	case ADD:
+	case SUB:
+	case CONCAT:
+	    _aggList =  _pExpr->getAggregationList() + _pTerm->getAggregationList();
+	    break;
+	case TERM:
+	    _aggList = _pTerm->getAggregationList();
+	    break;
+	}
+	_isAgg = true;
     }
+
+    return _aggList;
 }
 
 CegoExpr* CegoExpr::clone(bool isAttrRef)
@@ -323,8 +334,7 @@ CegoExpr* CegoExpr::clone(bool isAttrRef)
 	return ( new CegoExpr( _pExpr->clone(isAttrRef), _pTerm->clone(isAttrRef), _expType ) );	
     case TERM:
 	return ( new CegoExpr( _pTerm->clone(isAttrRef) ) );
-    }
-    
+    }    
 }    
 
 CegoExpr& CegoExpr::operator = ( const CegoExpr& e)
@@ -336,6 +346,27 @@ CegoExpr& CegoExpr::operator = ( const CegoExpr& e)
     return (*this);
 }
 
+Chain CegoExpr::getId() const
+{
+    Chain s;
+    switch ( _expType )
+    {
+    case CegoExpr::ADD:
+	s = _pExpr->getId() + "+" + _pTerm->getId();
+	break;
+    case CegoExpr::SUB:
+	s = _pExpr->getId() + "-" + _pTerm->getId();
+	break;
+    case CegoExpr::CONCAT:
+	s = _pExpr->getId() + "|" + _pTerm->getId();
+	break;
+    case CegoExpr::TERM:
+	s = _pTerm->getId();
+	break;
+    }
+    return s;
+}
+
 Chain CegoExpr::toChain(const Chain& indent) const
 {
     Chain s;
@@ -357,6 +388,11 @@ Chain CegoExpr::toChain(const Chain& indent) const
     return s;
 }
 
+Chain CegoExpr::dbFormat(CegoDatabaseFormater *pForm) const
+{
+    return pForm->formatExpr(_expType, _pExpr, _pTerm);
+}
+
 Element* CegoExpr::toElement() const
 {
     Element* pExprElement = new Element(XML_EXPR_ELEMENT);
@@ -488,7 +524,7 @@ void CegoExpr::encode(char *buf)
     }
 }
 
-void CegoExpr::decode(char *buf, CegoDistManager *pGTM)
+void CegoExpr::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
 {
 
     char* pE = (char*)buf;
@@ -502,16 +538,16 @@ void CegoExpr::decode(char *buf, CegoDistManager *pGTM)
     case CegoExpr::SUB:
     case CegoExpr::CONCAT:
     {
-	_pExpr = new CegoExpr(pE, pGTM);
+	_pExpr = new CegoExpr(pE, pGTM, tabSetId);
 	pE = pE + _pExpr->getEncodingLength();
-	_pTerm = new CegoTerm(pE, pGTM);
+	_pTerm = new CegoTerm(pE, pGTM, tabSetId);
 	pE = pE + _pTerm->getEncodingLength();
 	break;
     }
     case CegoExpr::TERM:
     {
 	_pExpr=0;
-	_pTerm = new CegoTerm(pE, pGTM);
+	_pTerm = new CegoTerm(pE, pGTM, tabSetId);
 	pE = pE + _pTerm->getEncodingLength();
 	break;
     }
@@ -561,8 +597,6 @@ void CegoExpr::getPlanList(ListT<Element*>& planList)
 
 ostream& operator << (ostream& s, const CegoExpr& e)
 {
-
     s << e.toChain();
     return s;
 }
-
