@@ -6,43 +6,29 @@
 //                                                 
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoPredDesc
-// 
-// Description: 
 //
-// Status: QG-2.6
+// Description: The CegoPrdDesc class is a container class for the query predicate representation
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoPredDesc.h"
 #include "CegoCondDesc.h"
 #include "CegoSelect.h"
 #include "CegoXMLdef.h"
-
+#include "CegoDatabaseFormater.h"
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
-CegoPredDesc::CegoPredDesc(char* buf, CegoDistManager *pGTM)
+CegoPredDesc::CegoPredDesc(char* buf, CegoDistManager *pGTM, int tabSetId)
 {
     _pExpr1 = 0;
     _pExpr2 = 0;
@@ -51,7 +37,7 @@ CegoPredDesc::CegoPredDesc(char* buf, CegoDistManager *pGTM)
     _pNotPred = 0;
     _isChecked = false;
     _pSelect = 0;
-    decode(buf, pGTM);
+    decode(buf, pGTM, tabSetId);
 }
 
 CegoPredDesc::CegoPredDesc(Element* pPredElement, CegoDistManager *pGTM)
@@ -314,7 +300,6 @@ void CegoPredDesc::getSelectQueryList(ListT<CegoSelect*>& queryList)
     {
 	_pNotPred->getSelectQueryList(queryList);
     }
-
 }
 
 CegoCondDesc* CegoPredDesc::getCondition()
@@ -352,7 +337,6 @@ CegoExpr* CegoPredDesc::getExpr3()
     return _pExpr3;
 }
 
-
 void CegoPredDesc::setBlock(CegoProcBlock* pBlock)
 {
     if ( _pExpr1 )
@@ -388,7 +372,6 @@ void CegoPredDesc::setChecked(bool val)
     _isChecked = val;
 }
 
-
 void CegoPredDesc::clearAttrCache()
 {
     if ( _pExpr1 )
@@ -415,8 +398,6 @@ void CegoPredDesc::clearAttrCache()
     {
 	_pNotPred->clearAttrCache();
     }
-
-
 }
 
 void CegoPredDesc::cleanUp()
@@ -454,10 +435,8 @@ bool CegoPredDesc::isChecked() const
     return _isChecked;
 }
 
-
 bool CegoPredDesc::hasOrCond() const
 {
-    
     if (_pC)
     {
 	if ( _pC->getCondType() == CegoCondDesc::OR )
@@ -467,9 +446,7 @@ bool CegoPredDesc::hasOrCond() const
 	else if ( _pC->getCondType() == CegoCondDesc::PRED )
 	    return _pC->Left()->hasOrCond();
     }
-
     return false;
-
 }
 
 const Chain& CegoPredDesc::getPattern() const
@@ -487,9 +464,8 @@ bool CegoPredDesc::match(const CegoFieldValue& val) const
 
 void CegoPredDesc::encode(char *buf)
 {
-    
     char* pP = (char*)buf;
-
+    
     memcpy( pP, &_mode, sizeof(CegoPredDesc::CompMode));
     pP = pP + sizeof(CegoPredDesc::CompMode);
 
@@ -515,7 +491,7 @@ void CegoPredDesc::encode(char *buf)
 	
 	_pExpr1->encode(pP);
 	pP = pP + _pExpr1->getEncodingLength();
-	
+
 	_pExpr2->encode(pP);
 	pP = pP + _pExpr2->getEncodingLength();
 
@@ -576,7 +552,7 @@ void CegoPredDesc::encode(char *buf)
     }
 }
 
-void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
+void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM, int tabSetId)
 {
     char* pP = (char*)buf;
 
@@ -595,13 +571,13 @@ void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
     {
     case CegoPredDesc::CONDITION:
     {
-	_pC = new CegoCondDesc(pP, pGTM);
+	_pC = new CegoCondDesc(pP, pGTM, tabSetId);
 	pP = pP + _pC->getEncodingLength();
 	break;
     }
     case CegoPredDesc::NOTPRED:
     {
-	_pNotPred = new CegoPredDesc(pP, pGTM);
+	_pNotPred = new CegoPredDesc(pP, pGTM, tabSetId);
 	pP = pP + _pNotPred->getEncodingLength();
 	break;
     }
@@ -611,10 +587,10 @@ void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
 	memcpy( &_comp, pP, sizeof(CegoComparison));
 	pP = pP + sizeof(CegoComparison);
 
-	_pExpr1 = new CegoExpr(pP, pGTM);
+	_pExpr1 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr1->getEncodingLength();
 
-	_pExpr2 = new CegoExpr(pP, pGTM);
+	_pExpr2 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr2->getEncodingLength();
 
 	break;
@@ -622,13 +598,13 @@ void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
     case CegoPredDesc::BETWEEN:
     {
 	
-	_pExpr1 = new CegoExpr(pP, pGTM);
+	_pExpr1 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr1->getEncodingLength();
 
-	_pExpr2 = new CegoExpr(pP, pGTM);
+	_pExpr2 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr2->getEncodingLength();
 
-	_pExpr3 = new CegoExpr(pP, pGTM);
+	_pExpr3 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr3->getEncodingLength();
 
 	break;
@@ -636,13 +612,13 @@ void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
     case CegoPredDesc::NULLCOMP:
     case CegoPredDesc::NOTNULLCOMP:
     {
-	_pExpr1 = new CegoExpr(pP, pGTM);
+	_pExpr1 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr1->getEncodingLength();
 	break;
     }
     case CegoPredDesc::EXISTSCOMP:
     {
-	_pSelect = new CegoSelect(pP, pGTM);
+	_pSelect = new CegoSelect(pP, pGTM, tabSetId);
 	pP = pP + _pSelect->getEncodingLength();
 	break;
     }
@@ -650,10 +626,10 @@ void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
     case CegoPredDesc::NOTINSUB:
     {
 	
-	_pExpr1 = new CegoExpr(pP, pGTM);
+	_pExpr1 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr1->getEncodingLength();
 
-	_pSelect = new CegoSelect(pP, pGTM);
+	_pSelect = new CegoSelect(pP, pGTM, tabSetId);
 	pP = pP + _pSelect->getEncodingLength();
 
 	break;
@@ -661,7 +637,7 @@ void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
     case CegoPredDesc::ISLIKE:
     case CegoPredDesc::ISNOTLIKE:
     {
-	_pExpr1 = new CegoExpr(pP, pGTM);
+	_pExpr1 = new CegoExpr(pP, pGTM, tabSetId);
 	pP = pP + _pExpr1->getEncodingLength();
 	
 	char len;
@@ -678,7 +654,6 @@ void CegoPredDesc::decode(char *buf, CegoDistManager *pGTM)
 
 int CegoPredDesc::getEncodingLength() const
 {
-
     int len = 0;
 
     len += sizeof(CegoPredDesc::CompMode);
@@ -813,9 +788,8 @@ ListT<CegoAttrDesc*> CegoPredDesc::getAttrRefList() const
 
 int CegoPredDesc::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl)
 {
-
     int refCount = 0;
-
+    
     switch (_mode )
     {
 
@@ -871,14 +845,12 @@ int CegoPredDesc::evalReferences(CegoContentObject *pCO, const ListT<CegoField>&
     }
 
     return refCount;
-
 }
 
 ListT<CegoField> CegoPredDesc::getFieldList() const
 {
-
     ListT<CegoField> fl;
-
+    
     switch (_mode )
     {
     case CegoPredDesc::CONDITION:
@@ -930,7 +902,6 @@ ListT<CegoField> CegoPredDesc::getFieldList() const
 
 CegoPredDesc* CegoPredDesc::clone(bool isAttrRef)
 {
-
     if ( _mode == CegoPredDesc::EXPRCOMP )
     {
 	return new CegoPredDesc(_pExpr1->clone(isAttrRef), _pExpr2->clone(isAttrRef), _comp);
@@ -976,7 +947,131 @@ CegoPredDesc* CegoPredDesc::clone(bool isAttrRef)
 	return new CegoPredDesc(_pExpr1->clone(isAttrRef), false);
     }
     throw Exception(EXLOC, "Clone not implemented");
+}
 
+Chain CegoPredDesc::getId() const
+{
+    Chain s;
+
+    switch (_mode )
+    {
+    case CegoPredDesc::CONDITION:
+    {
+
+	if (_pC->Left())
+	{
+	    s = Chain("( ");
+	    s += _pC->Left()->getId();
+
+	    switch(_pC->getCondType())
+	    {
+	    case CegoCondDesc::AND:
+		s += Chain("a");
+		break;
+	    case CegoCondDesc::OR:
+		s += Chain("o");
+		break;
+	    default:
+		break;
+	    }
+	    s += _pC->Right()->getId();
+	}
+	break;
+    }
+    case CegoPredDesc::NOTPRED:
+    {
+	s = Chain("n") + _pNotPred->getId();
+	break;
+    }
+    case CegoPredDesc::EXPRCOMP:
+    {
+	
+	s = _pExpr1->getId();
+	
+	switch (_comp)
+	{
+	case EQUAL:
+	    s += "=";
+	    break;
+	case NOT_EQUAL:
+	    s += "!=";
+	    break;
+	case LESS_THAN:
+	    s += "<";
+	    break;
+	case MORE_THAN:
+	    s += ">";
+	    break;
+	case LESS_EQUAL_THAN:
+	    s += "<=";
+	    break;
+	case MORE_EQUAL_THAN:
+	    s += ">=";
+	    break;
+	}
+	
+	s +=  _pExpr2->getId();
+	break;
+    }
+    case CegoPredDesc::BETWEEN:
+    {
+	
+	s = _pExpr2->getId();
+	s += Chain("bw");
+	s +=  _pExpr1->getId();
+	s += Chain("a");
+	s +=  _pExpr3->getId();
+
+	break;
+    }
+    case CegoPredDesc::ISLIKE:
+    {
+	s = _pExpr1->getId();
+	s += Chain("l");
+	s += Chain("'") + _pattern + Chain("'");
+	break;
+    }
+    case CegoPredDesc::ISNOTLIKE:
+    {
+	s = _pExpr1->getId();
+	s += Chain("nl");
+	s += Chain("'") + _pattern + Chain("'");
+	break;
+    }
+    case CegoPredDesc::NULLCOMP:
+    {
+	s =  _pExpr1->getId();
+	s += Chain("n");
+	break;
+    }
+    case CegoPredDesc::NOTNULLCOMP:
+    {
+	s = _pExpr1->getId();
+	s += Chain("nn");
+	break;
+    }
+    case CegoPredDesc::EXISTSCOMP:
+    {
+	s = Chain("e");
+	s += _pSelect->getQueryId();
+	break;
+    }
+    case CegoPredDesc::INSUB:
+    {
+	s = _pExpr1->getId();
+	s += Chain("i");
+	s += Chain("(") + _pSelect->getQueryId() + Chain(")");
+	break;
+    }
+    case CegoPredDesc::NOTINSUB:
+    {
+	s = _pExpr1->getId();
+	s += Chain("ni");
+	s += Chain("(") + _pSelect->getQueryId() + Chain(")");   
+	break;
+    }  
+    }
+    return s;
 }
 
 Chain CegoPredDesc::toChain(const Chain& indent) const
@@ -1106,9 +1201,13 @@ Chain CegoPredDesc::toChain(const Chain& indent) const
     return s;
 }
 
+Chain CegoPredDesc::dbFormat(CegoDatabaseFormater *pForm)
+{
+    return pForm->formatPred(_mode, _pExpr1, _pExpr2, _pExpr3, _comp, _pattern, _pC, _pNotPred, _pSelect);
+}
+    
 Element* CegoPredDesc::toElement() const
 {
-
     Element *pPredElement = new Element(XML_PRED_ELEMENT);
 
     switch (_mode )
@@ -1151,8 +1250,7 @@ Element* CegoPredDesc::toElement() const
 	case MORE_EQUAL_THAN:
 	    pPredElement->setAttribute( XML_COMP_ATTR, XML_MOREEQUALTHAN_VALUE );
 	    break;
-	}
-	
+	}	
 	break;
     }
     case CegoPredDesc::BETWEEN:
@@ -1215,7 +1313,6 @@ Element* CegoPredDesc::toElement() const
 
 void CegoPredDesc::fromElement(Element* pPredElement, CegoDistManager *pGTM)
 {
-
     if ( _pExpr1 )
 	delete _pExpr1;
     if ( _pExpr2 )
@@ -1318,8 +1415,6 @@ void CegoPredDesc::fromElement(Element* pPredElement, CegoDistManager *pGTM)
     }
     else if ( mode == Chain(XML_BETWEEN_VALUE) )
     {
-	
-
 	_mode = CegoPredDesc::BETWEEN;
 
 	ListT<Element*> el = pPredElement->getChildren(XML_EXPR_ELEMENT);
@@ -1367,7 +1462,6 @@ void CegoPredDesc::fromElement(Element* pPredElement, CegoDistManager *pGTM)
     }
     else if ( mode == Chain(XML_NULLCOMP_VALUE) )
     {
-
 	_mode = CegoPredDesc::NULLCOMP;
 	ListT<Element*> el = pPredElement->getChildren(XML_EXPR_ELEMENT);
 	Element **pEE = el.First();
