@@ -6,28 +6,13 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2008 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoQuery
 // 
-// Description: 
+// Description: All modifying queries
 //
 // Status: CLEAN
 //
@@ -39,7 +24,7 @@
 
 #include "CegoOutput.h"
 #include "CegoQuery.h"
-
+#include "CegoDatabaseFormater.h"
 
 CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableSet, CegoQuery::TransactionOp to)
 {
@@ -51,17 +36,14 @@ CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableSet, CegoQuery::Tr
 	_mode = ROLLBACK_QUERY;
     
     _pGTM = pGTM;
-    _pLogger = _pGTM->getDBMng();
     _tableSet = tableSet;
     _pPred = 0;
 }
 
-
 CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableName, const Chain& tableSet, const ListT<CegoField>& schema, const ListT<ListT<CegoExpr*> >& exprListArray)
 {
     _mode = INSERT_QUERY;
     _pGTM = pGTM;
-    _pLogger = _pGTM->getDBMng();
     _tableName = tableName;
     _tableSet = tableSet;
     _schema = schema;
@@ -70,12 +52,10 @@ CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableName, const Chain&
     _pPred = 0;
 }
 
-
 CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableName, const Chain& tableSet,  const ListT<CegoField>& schema, CegoSelect *pSelect)
 {
     _mode = INSERTBYSELECT_QUERY;
     _pGTM = pGTM;
-    _pLogger = _pGTM->getDBMng();
     _tableName = tableName;
     _tableSet = tableSet;
     _schema = schema;
@@ -83,12 +63,10 @@ CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableName, const Chain&
     _pPred = 0;
 }
 
-
 CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableName,  const Chain& tableAlias, const Chain& tableSet, CegoPredDesc* pPred)
 {
     _mode = DELETE_QUERY;
     _pGTM = pGTM;
-    _pLogger = _pGTM->getDBMng();
     _tableName = tableName;
     _tableAlias = tableAlias;
     _tableSet = tableSet;
@@ -100,7 +78,6 @@ CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableName, const Chain&
 {
     _mode = UPDATE_QUERY;
     _pGTM = pGTM;
-    _pLogger = _pGTM->getDBMng();
     _tableName = tableName;
     _tableAlias = tableAlias;
     _tableSet = tableSet;
@@ -115,7 +92,6 @@ CegoQuery::CegoQuery(CegoDistManager* pGTM, const Chain& tableName,  const Chain
 {
     _mode = ALTER_QUERY;
     _pGTM = pGTM;
-    _pLogger = _pGTM->getDBMng();
     _tableName = tableName;
     _tableSet = tableSet;
     _alterList = alterList;
@@ -127,7 +103,6 @@ CegoQuery::CegoQuery(CegoDistManager* pGTM,   const Chain& tableSet, const Chain
 {
     _mode = RENAME_QUERY;
     _pGTM = pGTM;
-    _pLogger = _pGTM->getDBMng();
     _tableSet = tableSet;
     _objName = objName;
     _objType = objType;
@@ -136,7 +111,6 @@ CegoQuery::CegoQuery(CegoDistManager* pGTM,   const Chain& tableSet, const Chain
     _pSelect = 0;
 }
 
-
 CegoQuery::~CegoQuery()
 {
 
@@ -208,6 +182,9 @@ void CegoQuery::setBlock(CegoProcBlock *pBlock)
 
 Chain CegoQuery::execute(CegoProcBlock *pBlock)
 {
+    
+    if ( _pGTM == 0 )
+	throw Exception(EXLOC, "No valid table manager set up");
 
     _affCount = 0;
 
@@ -305,7 +282,6 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		    {
 			_schema.Insert( *pF );
 			
-
 			ListT<CegoExpr*> *pExprList = _exprListArray.First();
 			while ( pExprList )
 			{			    
@@ -359,27 +335,23 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	    }
 	}
 	
-
 	ListT<CegoExpr*> *pExprList = _exprListArray.First();
 	while ( pExprList )
 	{	   
 	    CegoField* pFV = _schema.First();
 	    CegoExpr** pExpr = pExprList->First();
 	    
-	    int pos=1;
 	    while ( pFV && pExpr)
 	    {	    
 		
 		(*pExpr)->setBlock(pBlock);
 		CegoFieldValue fv = (*pExpr)->evalFieldValue();
 		
-		CegoQueryHelper qh;
-		qh.prepareFieldValue(pFV, fv, pos);
+		CegoQueryHelper::prepareFieldValue(pFV, fv, _pGTM, oe.getTabSetId());
 		
 		pFV->setValue(fv); 
 		pFV = _schema.Next();
-		pExpr = pExprList->Next();	    
-		pos++;
+		pExpr = pExprList->Next();
 	    }
 	    if ( pFV || pExpr )
 	    {
@@ -388,7 +360,11 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 
 	    try 
 	    {
+				
 		_pGTM->insertDistDataTable(oe, _schema);
+
+		_pGTM->getDBMng()->cleanCache( oe.getTabSetId(), CegoObject::TABLE, oe.getTabName());
+		
 		_affCount++;
 	    }
 	    catch ( Exception e )
@@ -429,8 +405,9 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	    throw CegoQueryException(INVALID_OBJECT_EXCEP, EXLOC, msg);    
 	}
 	
-	try {
-
+	try
+	{
+	    
 	    if (  _schema.isEmpty() )
 	    {   
 		_schema = oe.getSchema();
@@ -486,14 +463,12 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		    pF = _schema.Next();
 		}
 	    }
-	    
-	    
+	    	    
 	    int tabSetId = _pGTM->getDBMng()->getTabSetId(_tableSet);    
 	    
 	    _pSelect->setProcBlock(pBlock);
 	    _pSelect->setTabSetId(tabSetId);
 	    _pSelect->prepare();
-	    // _pSelect->getSchema(schema);
 	    
 	    ListT<CegoField> fl;
 	    bool moreTuple = _pSelect->nextTuple(fl);
@@ -504,19 +479,16 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		CegoField* pFV = insertSchema.First();
 		CegoField* pSF = fl.First();
 		
-		int pos=1;
 		while ( pFV && pSF)
 		{
 		    
-		    CegoQueryHelper qh;
 		    CegoFieldValue fv = pSF->getValue();
-
-		    qh.prepareFieldValue(pFV, fv, pos);
+		    
+		    CegoQueryHelper::prepareFieldValue(pFV, fv, _pGTM, oe.getTabSetId());
 		    
 		    pFV->setValue(fv); 
 		    pFV = insertSchema.Next();
 		    pSF = fl.Next();	    
-		    pos++;
 		}
 
 		_pGTM->insertDistDataTable(oe, insertSchema);
@@ -525,6 +497,9 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		moreTuple = _pSelect->nextTuple(fl);
 	    }
 	    _pSelect->cleanUp();
+
+	    _pGTM->getDBMng()->cleanCache( oe.getTabSetId(), CegoObject::TABLE, oe.getTabName());
+	    
 	}
 	catch ( Exception e )
 	{
@@ -553,7 +528,6 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	try 
 	{	    
 	    _pGTM->getDistObject(_tableSet, _tableName, CegoObject::TABLE, oe);
-
 	}
 	catch ( Exception e )
 	{
@@ -563,9 +537,10 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	}
 
 	try 
-	{
+	{	    
 	    oe.setTabAlias(_tableAlias);
 	    _affCount = _pGTM->deleteDistDataTable(oe, _pPred, pBlock);
+	    _pGTM->getDBMng()->cleanCache(oe.getTabSetId(), CegoObject::TABLE, oe.getTabName() );
 	}
 	catch ( Exception e )
 	{
@@ -600,10 +575,18 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	    e.pop(msg);
 	    throw CegoQueryException(INVALID_OBJECT_EXCEP, EXLOC, msg);    
 	}
+
+	oe.setTabAlias(_tableAlias);
 	
 	CegoField *pF = _schema.First();
 	while ( pF )
 	{
+	    
+	    if ( _tableName != _tableAlias )
+	    {	    
+		pF->setTableAlias(_tableAlias);
+	    }
+
 	    CegoField *pUF = oe.getSchema().Find(*pF);
 	    if ( pUF )
 	    {		
@@ -622,13 +605,11 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	
 	try 
 	{
-	    oe.setTabAlias(_tableAlias);
-	   _affCount =  _pGTM->updateDistDataTable(oe, _pPred, _schema, _updList,  pBlock);
-
+	    _affCount =  _pGTM->updateDistDataTable(oe, _pPred, _schema, _updList,  pBlock);
+	    _pGTM->getDBMng()->cleanCache( oe.getTabSetId(), CegoObject::TABLE, oe.getTabName());
 	}
 	catch ( Exception e )
 	{
-
 	    Chain msg;
 	    e.pop(msg);
 	    throw CegoQueryException(COREOP_EXCEP, EXLOC, msg);	    
@@ -638,13 +619,15 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	while ( pRV ) 
 	{	    
 	    if ( pBlock )
-	    {
-		
+	    {	
 		CegoExpr *pExpr = (*pRV)->getExpr();
 
-		pExpr->setFieldListArray(&_schema, 1);
-		CegoFieldValue fv = pExpr->evalFieldValue();
+		// we need to create an additional reference to use the setFieldListArray method
+		ListT<CegoField>* fl[1];
+		fl[0] = &_schema;
 
+		pExpr->setFieldListArray(fl);
+		CegoFieldValue fv = pExpr->evalFieldValue();
 		pBlock->setValue((*pRV)->getVarName(), fv); 
 	    }
 	    pRV = _retVarList.Next();
@@ -667,9 +650,7 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 
 	try
 	{
-
 	    _pGTM->getDistObject(_tableSet, _tableName, CegoObject::TABLE, oe);
-
 	}
 	catch ( Exception e )
 	{
@@ -680,10 +661,9 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 
 	try 
 	{
-	
 	    _pGTM->alterDistDataTable(oe, _alterList);
 	    _affCount++;
-
+	    _pGTM->getDBMng()->cleanCache(oe.getTabSetId(), CegoObject::TABLE, oe.getTabName());
 	}
 	catch ( Exception e )
 	{
@@ -697,8 +677,6 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
     }
     case RENAME_QUERY:
     {
-
-
 	if ( _pGTM->getAutoCommit() == false )
 	{
 	    if ( _pGTM->getDistTid(_tableSet) == 0 )
@@ -706,13 +684,28 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 		_pGTM->startDistTransaction(_tableSet);
 	    }
 	}
+
+	if ( _objType == CegoObject::TABLE )
+	{
+	    CegoTableObject oe;
+	    
+	    try
+	    {		
+		_pGTM->getDistObject(_tableSet, _objName, _objType, oe);	
+	    }
+	    catch ( Exception e )
+	    {
+		Chain msg;
+		e.pop(msg);
+		throw CegoQueryException(INVALID_OBJECT_EXCEP, EXLOC, msg);    
+	    }
+	    _pGTM->getDBMng()->cleanCache(oe.getTabSetId(), CegoObject::TABLE, oe.getTabName() );	    
+	}
 	
 	try 
 	{
-
 	    _pGTM->renameDistObject(_tableSet, _objName, _objType, _newObjName);
 	    _affCount++;
-
 	}
 	catch ( Exception e )
 	{
@@ -721,7 +714,6 @@ Chain CegoQuery::execute(CegoProcBlock *pBlock)
 	    throw CegoQueryException(COREOP_EXCEP, EXLOC, msg);   
 	}
 	return Chain("Object renamed");	
-
     }
     }
 }
@@ -752,9 +744,7 @@ Chain CegoQuery::toChain(const Chain& indent) const
 	break;
     }
     case INSERT_QUERY:
-    {
-
-	
+    {	
 	s = indent + Chain("insert into ") + _tableName;
 
 	if ( _schema.Size() > 0 )
@@ -798,7 +788,6 @@ Chain CegoQuery::toChain(const Chain& indent) const
     }
     case INSERTBYSELECT_QUERY:
     {
-
 	s = indent + Chain("insert into ") + _tableName;
 
 	if ( _schema.Size() > 0 )
@@ -838,7 +827,6 @@ Chain CegoQuery::toChain(const Chain& indent) const
     }
     case UPDATE_QUERY:
     {
-
 	s = indent + Chain("update ") + _tableName;
 
 	if ( _tableName != _tableAlias )
@@ -866,7 +854,6 @@ Chain CegoQuery::toChain(const Chain& indent) const
 	    s += _pPred->toChain();
 	}
 
-	
 	if ( _retVarList.isEmpty() == false )
 	{ 
 	    s += Chain(" return ");
@@ -905,7 +892,7 @@ Chain CegoQuery::toChain(const Chain& indent) const
 	    s += Chain("table ");
 	    break;
 	}
-	case CegoObject::INDEX:
+	case CegoObject::AVLTREE:
 	{
 	    s += Chain("index ");
 	    break;
@@ -933,7 +920,10 @@ Chain CegoQuery::toChain(const Chain& indent) const
 	break;
     }
     }
+    return s;    
+}
 
-    return s;
-    
+Chain CegoQuery::dbFormat(CegoDatabaseFormater *pForm) const
+{
+    return pForm->formatQuery(_mode, _tableName, _tableAlias, _schema,  _exprListArray,  _updList, _retVarList, _alterList, _pPred, _pSelect, _objName, _objType, _newObjName);
 }
