@@ -987,8 +987,7 @@ void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, Cego
 	    Chain shrinkVal ( (char*) fv.getValue() );
 	    fv = CegoFieldValue(VARCHAR_TYPE, shrinkVal.subChain(1, pFV->getLength()));
 	}
-	
-	if ( pFV->getType() == DATETIME_TYPE )
+	else if ( pFV->getType() == DATETIME_TYPE )
 	{
 	    int val;
 	    memcpy(&val, fv.getValue(), sizeof(int));
@@ -999,8 +998,7 @@ void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, Cego
 		memcpy(fv.getValue(), &dtval, sizeof(int));	   	    
 	    }
 	}
-
-	if ( pFV->getType() == BLOB_TYPE || pFV->getType() == CLOB_TYPE )
+	else if ( pFV->getType() == BLOB_TYPE || pFV->getType() == CLOB_TYPE )
 	{
 	    int fileId;
 	    int pageId;
@@ -1035,6 +1033,25 @@ void CegoQueryHelper::prepareFieldValue(CegoField* pFV, CegoFieldValue& fv, Cego
 		throw Exception(EXLOC, Chain("Invalid lob value"));
 	    }
 	}
+	else if ( pFV->getType() == FIXED_TYPE )
+	{
+
+	    // for fixed values, we have to check the correct dimension
+	    
+	    Chain fixedVal = fv.valAsChain();
+	    
+	    int pos;
+	    if ( fixedVal.posStr(Chain("."), pos) == false )
+		throw Exception(EXLOC, "Invalid fixed value");
+	    
+	    int dim =  fixedVal.length() - pos - 1;
+
+	    if ( dim != pFV->getLength() )
+	    {
+		throw Exception(EXLOC, Chain("Wrong dimension for fixed value"));
+	    }
+	    
+	}
     }
 }
 
