@@ -6,41 +6,25 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //
-// (C)opyright 2000-2012 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoRecoveryManager
 // 
-// Description: 
+// Description: Transaction recovery handling
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// base includes
+// LFC INCLUDES
 #include <lfcbase/SetT.h>
 #include <lfcbase/Datetime.h>
 #include <lfcbase/Sleeper.h>
-#include <lfcbase/Host.h>
 #include <lfcbase/CommandExecuter.h>
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoXMLdef.h"
 #include "CegoRecoveryManager.h"
 
@@ -60,7 +44,7 @@ CegoRecoveryManager::~CegoRecoveryManager()
 {
 }
 
-long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
+unsigned long long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
 {   
 
     _pLogger->log(_modId, Logger::NOTICE, Chain("Recovering tableset ") + tableSet + Chain(" ..."));
@@ -109,12 +93,10 @@ long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
 	if ( _recoveryMode == LOCAL )
 	    _pDBMng->releaseLogFiles(tableSet, true);
 
-
 	// during recovery, we do not write to online log files
 	_pDBMng->stopLog(tabSetId);	
-	
-	
-	long cplsn;
+		
+	unsigned long long cplsn;
 	
 	Chain tsTicketName = _pDBMng->getTSTicket(tableSet);
 	
@@ -143,7 +125,7 @@ long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
 	    cplsn = _pDBMng->getCommittedLSN(tableSet);
 	    
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Starting datafile recovery for tableset ") + tableSet + Chain(" ..."));
-	    long lsn = dataFileRecovery(tableSet, tabSetId, cplsn);
+	    unsigned long long lsn = dataFileRecovery(tableSet, tabSetId, cplsn);
 	    
 
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Datafile recovery for tableset ") + tableSet + Chain(" finished with lsn ") + Chain(lsn));
@@ -168,22 +150,17 @@ long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
 	{
 	    Datetime d(pit);
 	    _pDBMng->log(_modId, Logger::NOTICE, 
-			 Chain("Starting point-in-time transaction recovery to ") + d.asChain() + Chain(" for tableset ") + tableSet + Chain(" ..."));
-	    
+			 Chain("Starting point-in-time transaction recovery to ") + d.asChain() + Chain(" for tableset ") + tableSet + Chain(" ..."));	    
 	}
 	
 	_pDBMng->log(_modId, Logger::NOTICE, Chain("Commited lsn =  ") + Chain(cplsn));
 	
-	long lsn = transactionRecovery(tableSet, tabSetId, cplsn, pit);
-	
-#ifdef CGDEBUG	
-	_pDBMng->log(_modId, Logger::DEBUG, Chain("Transaction recovery finished with lsn = ") + Chain(lsn));
-#endif
+	unsigned long long lsn = transactionRecovery(tableSet, tabSetId, cplsn, pit);
 	
 	_pDBMng->initLogFiles(tableSet, true);
 	
 	_pDBMng->setActiveLogFile(tableSet);    
-	_pDBMng->setLSN(tabSetId, lsn + 1);
+
 	_pDBMng->startLog(tabSetId);
         
 	CegoLogRecord logRec;    
@@ -197,14 +174,11 @@ long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
 	_pDBMng->setTableSetRunState(tableSet, XML_ONLINE_VALUE);    
 	
 	_pDBMng->setRecoveryMode(tabSetId, CegoDatabaseManager::OFF );
+
+	unsigned long long newcplsn = _pDBMng->getLSN(tabSetId);
 	
-	long newcplsn = _pDBMng->getLSN(tabSetId) - 1;
 	_pDBMng->setCommittedLSN(tabSetId, newcplsn);
-	
-#ifdef CGDEBUG	
-	_pDBMng->log(_modId, Logger::DEBUG, Chain("Writing sync point ..."));
-#endif
-	
+		
 	_pGTM->registerObjects(tableSet);
 	
 	_pGTM->writeCheckPoint(tableSet, true, false);
@@ -225,12 +199,12 @@ long CegoRecoveryManager::recoverTableSet(const Chain& tableSet, int pit)
     }
 }
 
-long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId, long cplsn)
+unsigned long long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId, unsigned long long cplsn)
 {
 
     bool endOfBackup = false;       
    
-    long lsn = cplsn;
+    unsigned long long lsn = cplsn;
 
     while ( endOfBackup == false &&  _pDBMng->getRecoveryMode(tabSetId) == CegoDatabaseManager::ON )
     {
@@ -271,15 +245,12 @@ long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId,
 
 	if ( fileFound )
 	{
-
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Recovering offline logfile ") + sourceFileName + Chain(" ..."));
-
 	    _pDBMng->setLogFile(tabSetId, sourceFileName, true);
 	    _pDBMng->setLSN(tabSetId, lsn + 1);
 	    
 	    // recover offline logs
-	    lsn = recoverCurrentDataFileLog(tabSetId, endOfBackup);
-	    
+	    lsn = recoverCurrentDataFileLog(tabSetId, endOfBackup);	   
 	}
 	else if ( _recoveryMode == LOCAL )
 	{
@@ -289,6 +260,7 @@ long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId,
     
 	    if ( archRestoreProg != Chain("NONE") )
 	    {
+
 		
 		Chain *pArchLogPath = archPathList.First();
 		Chain pathString;
@@ -304,7 +276,7 @@ long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId,
 		
 		int archRestoreTimeout = _pDBMng->getArchRestoreTimeout();
 		
-		Chain restoreCmd = archRestoreProg + Chain(" ") + archLogFileName + Chain(" ") + pathString; 		
+		Chain restoreCmd = archRestoreProg + Chain(" ") + tableSet + Chain(" ") + archLogFileName + Chain(" ") + pathString; 		
 		_pDBMng->log(_modId, Logger::NOTICE, Chain("Triggering external log manager with <") + Chain(restoreCmd) + Chain(">"));
 		int retCode = cmdExe.execute(restoreCmd, archRestoreTimeout);
 		_pDBMng->log(_modId, Logger::NOTICE, Chain("External log manager returned : <") + Chain(retCode) + Chain(">"));
@@ -338,13 +310,13 @@ long CegoRecoveryManager::dataFileRecovery(const Chain& tableSet, int tabSetId,
     return lsn;
 }
 
-long CegoRecoveryManager::transactionRecovery(const Chain& tableSet, int tabSetId, long cplsn, int pit)
+unsigned long long CegoRecoveryManager::transactionRecovery(const Chain& tableSet, int tabSetId, unsigned long long cplsn, int pit)
 {
     bool endOfRecovery = false;    
     Chain sourceFileName;
     int ts = 0;
 
-    long lsn = cplsn;
+    unsigned long long lsn = cplsn;
 
     RecoveryState rstate = CegoRecoveryManager::RECOK;
 	    
@@ -407,13 +379,12 @@ long CegoRecoveryManager::transactionRecovery(const Chain& tableSet, int tabSetI
 			if ( pArchLogPath )
 			    pathString += Chain(":");
 		    }
-		    
-		    
+		    		    
 		    CommandExecuter cmdExe(_shellCmd);
 		    
 		    int archRestoreTimeout = _pDBMng->getArchRestoreTimeout();
 		    
-		    Chain restoreCmd = archRestoreProg + Chain(" ") + archLogFileName + Chain(" ") + pathString; 
+		    Chain restoreCmd = archRestoreProg + Chain(" ") + tableSet + Chain(" ") + archLogFileName + Chain(" ") + pathString; 
 		    
 		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Triggering external log manager with <") + Chain(restoreCmd) + Chain(">"));
 		    int retCode = cmdExe.execute(restoreCmd, archRestoreTimeout); 
@@ -482,8 +453,6 @@ long CegoRecoveryManager::transactionRecovery(const Chain& tableSet, int tabSetI
 		}
 	    }
 	}
-
-
     }
 
     _pDBMng->log(_modId, Logger::NOTICE, Chain("Finished recovery loop ..."));
@@ -508,13 +477,14 @@ long CegoRecoveryManager::transactionRecovery(const Chain& tableSet, int tabSetI
 
     _pGTM->writeCheckPoint(tableSet, false, false);
 
+    _pDBMng->log(_modId, Logger::NOTICE, Chain("Transaction recovery finished with lsn ") + Chain(lsn));
     return lsn;
 }
 
 int CegoRecoveryManager::recoverCurrentDataFileLog(int tabSetId, bool& endOfBackup)
 {
     
-    long lsn=0;
+    unsigned long long lsn=0;
 
     _pDBMng->seekToStart(tabSetId);
 
@@ -575,25 +545,16 @@ int CegoRecoveryManager::recoverCurrentDataFileLog(int tabSetId, bool& endOfBack
 	{
 	    _pDBMng->log(_modId, Logger::NOTICE, Chain("Skipping logfile lsn ") + Chain(logRec.getLSN()) + Chain(" ( expected ") + Chain(_pDBMng->getLSN(tabSetId)) + Chain(")"));	    
 	    endOfBackup=true;
-
 	}
-
 	if ( logRec.getData() )
-	    free ( logRec.getData() ) ;
-	
+	    free ( logRec.getData() ) ;	
     }
-    
-    _pDBMng->log(_modId, Logger::NOTICE, Chain("Datafile recovery finished with lsn ") + Chain(lsn));
-   
     return lsn;
-    
 }
 
-CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactionLog(int tabSetId, int pit, long& lsn, int& ts)
+CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactionLog(int tabSetId, int pit, unsigned long  long& lsn, int& ts)
 {
     
-    // lsn=0;
-
     // initially, we set the logfile to incomplete
     RecoveryState rstate = CegoRecoveryManager::RECINCOMPLETE;
 
@@ -637,9 +598,7 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		ts=logRec.getTS();
 		int tid=logRec.getTID();
 
-#ifdef CGDEBUG	
-		_pDBMng->log(_modId, Logger::DEBUG, Chain("Recovering lsn ") + Chain(logRec.getLSN()) + Chain(" ..."));
-#endif
+		_pDBMng->log(_modId, Logger::NOTICE, Chain("Recovering lsn ") + Chain(logRec.getLSN()) + Chain(" ..."));
 
 		// here comes the recovery
 		
@@ -647,7 +606,7 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		{
 		    
 		case CegoLogRecord::LOGREC_CREATE:
-		    
+		{
 		    switch ( logRec.getObjType() )
 		    {
 			
@@ -657,15 +616,27 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			CegoTableObject to;
 			to.decode(logRec.getData());
 			_pGTM->createDataTable(tabSetId, logRec.getObjName(), logRec.getObjType(), to.getSchema());
+			_pDBMng->addObject(tabSetId, logRec.getObjName(), CegoObject::TABLE);
 			break;
 		    }
-		    case CegoObject::INDEX:
-		    case CegoObject::PINDEX:
-		    case CegoObject::UINDEX:
+		    case CegoObject::AVLTREE:
+		    case CegoObject::PAVLTREE:
+		    case CegoObject::UAVLTREE:
 		    {
 			CegoTableObject to;
 			to.decode(logRec.getData());
 			_pGTM->createIndexTable(tabSetId, logRec.getObjName(), to.getTabName(), to.getSchema(), logRec.getObjType());
+			_pDBMng->addObject(tabSetId, logRec.getObjName(), logRec.getObjType());
+			break;
+		    }
+		    case CegoObject::BTREE:
+		    case CegoObject::PBTREE:
+		    case CegoObject::UBTREE:
+		    {
+			CegoBTreeObject bo;
+			bo.decode(logRec.getData());
+			_pGTM->createIndexTable(tabSetId, logRec.getObjName(), bo.getTabName(), bo.getSchema(), logRec.getObjType());
+			_pDBMng->addObject(tabSetId, logRec.getObjName(), logRec.getObjType());
 			break;
 		    }
 		    case CegoObject::VIEW:
@@ -673,6 +644,7 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			CegoViewObject vo;
 			vo.decode(logRec.getData());
 			_pGTM->createViewObject(vo);
+			_pDBMng->addObject(tabSetId, logRec.getObjName(), CegoObject::VIEW);
 			break;		    
 		    }
 		    case CegoObject::PROCEDURE:
@@ -680,6 +652,7 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			CegoProcObject po;
 			po.decode(logRec.getData());
 			_pGTM->createProcObject(po);
+			_pDBMng->addObject(tabSetId, logRec.getObjName(), CegoObject::PROCEDURE);
 			break;		    
 		    }
 		    case CegoObject::FKEY:
@@ -687,15 +660,26 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			CegoKeyObject ko;
 			ko.decode(logRec.getData());
 			_pGTM->createKeyObject(ko);
+			_pDBMng->addObject(tabSetId, logRec.getObjName(), CegoObject::FKEY);
+			break;
+		    }
+		    case CegoObject::CHECK:
+		    {
+			CegoCheckObject co;
+			co.decode(logRec.getData());
+			_pGTM->createCheckObject(co);
+			_pDBMng->addObject(tabSetId, logRec.getObjName(), CegoObject::CHECK);
 			break;
 		    }
 		    default:		    
 			break;
 		    }
 		    break;
+		}
 		case CegoLogRecord::LOGREC_DROP:
 		{
 		    _pGTM->removeObject(tabSetId, logRec.getObjName(), logRec.getObjType());
+		    _pDBMng->removeObject(tabSetId, logRec.getObjName(), logRec.getObjType());
 		    break;
 		}
 		case CegoLogRecord::LOGREC_ALTER:
@@ -709,6 +693,8 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		{
 		    Chain newObjName(logRec.getData(), logRec.getDataLen());
 		    _pGTM->renameObject(tabSetId, logRec.getObjName(), logRec.getObjType(), newObjName );
+		    _pDBMng->removeObject(tabSetId, logRec.getObjName(), logRec.getObjType());
+		    _pDBMng->addObject(tabSetId, newObjName, logRec.getObjType());
 		    break;   
 		}
 		case CegoLogRecord::LOGREC_BEGIN:
@@ -717,7 +703,7 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		    break;
 		}
 		case CegoLogRecord::LOGREC_COMMIT:
-		{
+		{		    
 		    _pGTM->setTID(tabSetId, tid);		    
 		    _pGTM->commitTransaction(tabSetId);
 		    break;
@@ -736,18 +722,34 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 
 		    if ( invalidateSet.Find(logRec.getObjName()) == 0 )
 		    {
-			_pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index for table ") + logRec.getObjName());		    
+			_pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index for table ") + logRec.getObjName());
 			_pGTM->invalidateIndexForTable(tabSetId, logRec.getObjName());
 			invalidateSet.Insert(logRec.getObjName());
 		    }
 		    ListT<CegoField> fvl = oe.getSchema();
 		    
-		    int rectid = 0;
-		    int tastep = 0;
-		    CegoTupleState ts;
 		    ListT<CegoBlob> blobList;
-		    CegoQueryHelper qh;
-		    qh.decodeFVL(fvl, blobList, logRec.getData(), logRec.getDataLen(), rectid, tastep, ts);
+		    ListT<CegoClob> clobList;
+
+
+#ifdef CGDEBUG	
+		    _pDBMng->log(_modId, Logger::DEBUG, Chain("Decoding log record of len ") + Chain(logRec.getDataLen()));
+#endif
+		    
+		    unsigned long long tid;
+		    unsigned long long tastep;
+		    CegoTupleState ts;
+		    
+		    int toff = CegoQueryHelper::decodeTupleHeader(tid, tastep, ts, logRec.getData());
+		    
+		    char* tp =  logRec.getData() + toff;
+		    int tlen = logRec.getDataLen() - toff;
+
+		    CegoQueryHelper::decodeFVL(fvl, blobList, clobList, tp, tlen);
+
+#ifdef CGDEBUG	
+		    _pDBMng->log(_modId, Logger::DEBUG, Chain("Log record decoded"));
+#endif
 		    
 		    CegoBlob *pBlob = blobList.First();
 		    CegoField *pF = fvl.First();
@@ -777,23 +779,50 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			pBlob = blobList.Next();
 
 		    }
+
+		    CegoClob *pClob = clobList.First();
+		    pF = fvl.First();
+		    while ( pClob && pF )
+		    {
+	    	    
+			int fileId;
+			int pageId;
+			
+			_pGTM->putClobData(tabSetId, pClob->getBufPtr(), pClob->getSize(), fileId, pageId);
+			
+			while ( pF->getValue().getType() != CLOB_TYPE && pF)
+			    pF = fvl.Next();
+			
+			if ( pF )
+			{
+			    Chain clobRef = Chain("[") + Chain(fileId) + Chain(",") + Chain(pageId) + Chain("]");
+			    CegoFieldValue fv(CLOB_TYPE, clobRef);
+			    pF->setValue(fv);
+			}
+			else
+			{
+			    throw Exception(EXLOC, "Cannot get clob reference");
+			}
+			
+			free ( pClob->getBufPtr() );
+			pClob = clobList.Next();
+
+		    }
 		    
 		    bool lastPage = true;
 		    CegoDataPointer dp;
 
 		    _pGTM->setTID(tabSetId, tid);
 		    _pGTM->insertDataTable(oe, fvl, dp, lastPage);
-		    
 		    break;
 		}
 		case CegoLogRecord::LOGREC_DELETE:
 		{
 		  
-		    CegoPredDesc *pPred = 0;	    
-		    CegoQueryHelper qh;
+		    CegoPredDesc *pPred = 0;
 		    Chain tableAlias;
 
-		    qh.decodeDelRec(tableAlias, pPred, logRec.getData(), logRec.getDataLen(), _pGTM);
+		    CegoQueryHelper::decodeDelRec(tableAlias, pPred, logRec.getData(), logRec.getDataLen(), _pGTM, tabSetId);
 
 		    _delRecList.Insert(DeleteRecord(tid, tableAlias, pPred));
 		    
@@ -812,13 +841,13 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			
 			if ( invalidateSet.Find(logRec.getObjName()) == 0 )
 			{
-			    _pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index for table ") + logRec.getObjName());		    
+			    _pDBMng->log(_modId, Logger::NOTICE, Chain("Invalidating index for table ") + logRec.getObjName());
 			    _pGTM->invalidateIndexForTable(tabSetId, logRec.getObjName());
 			    invalidateSet.Insert(logRec.getObjName());
 			}
 			
 			_pGTM->setTID(tabSetId, tid);
-			_pGTM->deleteDataTable(oe, delRecord->getPred(), 0);		    
+			_pGTM->deleteDataTable(oe, delRecord->getPred(), 0);
 
 			delRecord->clean();
 			_delRecList.Remove(DeleteRecord(tid));
@@ -839,9 +868,9 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 		    ListT<CegoExpr*> exprList;
 
 		    CegoPredDesc *pPred = 0;		    
-		    CegoQueryHelper qh;
 		    Chain tableAlias;
-		    qh.decodeUpdRec(tableAlias, pPred, updList, exprList, logRec.getData(), logRec.getDataLen(), _pGTM);
+		    
+		    CegoQueryHelper::decodeUpdRec(tableAlias, pPred, updList, exprList, logRec.getData(), logRec.getDataLen(), _pGTM, tabSetId);
 
 		    _updRecList.Insert(UpdateRecord(tid, tableAlias, pPred, updList, exprList));
 
@@ -862,15 +891,16 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 			    invalidateSet.Insert(logRec.getObjName());
 			}
 
-			_pGTM->setTID(tabSetId,tid); 
+			_pGTM->setTID(tabSetId,tid);
 			_pGTM->updateDataTable(tabSetId, logRec.getObjName(),
 					       updRecord->getTable(), 
 					       updRecord->getPred(), 
 					       updRecord->getUpdateList(), 
 					       updRecord->getExprList(), 0);
-			
+
 			updRecord->clean();
 			_updRecList.Remove(UpdateRecord(tid));
+			
 		    }
 		    else
 		    {
@@ -922,10 +952,8 @@ CegoRecoveryManager::RecoveryState CegoRecoveryManager::recoverCurrentTransactio
 
     }
     
-    _pDBMng->log(_modId, Logger::NOTICE, Chain("Recovery finished with lsn ") + Chain(lsn));
+    _pDBMng->log(_modId, Logger::NOTICE, Chain("Current logfile recovery finished with lsn ") + Chain(lsn));
 
     return rstate;
     
 }
-
-
