@@ -287,6 +287,12 @@ CegoSelect::~CegoSelect()
 	_pNativeOrderList = 0;	
     }
 
+    if ( _pNativeOrderOptList )
+    {
+	delete _pNativeOrderOptList;
+	_pNativeOrderOptList = 0;	
+    }
+    
     if ( _pGroupList )
     {
 	if ( _pGroupCursor )
@@ -1068,13 +1074,19 @@ Chain CegoSelect::getQueryId() const
 	}
     }
 
-    if ( _pNativeOrderList )
+    if ( _pNativeOrderList && _pNativeOrderOptList )
     {
 	CegoExpr **pExpr = _pNativeOrderList->First();
-	while ( pExpr )
+	CegoOrderNode::Ordering *pOrdering = _pNativeOrderOptList->First();
+	while ( pExpr && pOrdering )
 	{
 	    id += (*pExpr)->getId();
 	    id += Chain("#");
+	    if ( *pOrdering == CegoOrderNode::ASC )
+		id += Chain("A");
+	    else
+		id += Chain("D");
+	    id += Chain("#");
 	    pExpr = _pNativeOrderList->Next();
 	}
     }
@@ -3466,9 +3478,16 @@ Chain CegoSelect::toChain(const Chain& indent) const
 	s += indent + Chain("order by\n") + indent;
 	
 	CegoExpr **pExpr = _pNativeOrderList->First();
-	while ( pExpr )
+	CegoOrderNode::Ordering *pOrdering = _pNativeOrderOptList->First();
+	while ( pExpr && pOrdering )
 	{
 	    s += (*pExpr)->toChain(indent + Chain(" "));
+
+	    if ( *pOrdering == CegoOrderNode::ASC )
+		s += Chain(" asc");
+	    else
+		s += Chain(" desc");
+	    
 	    pExpr = _pNativeOrderList->Next();
 	    if ( pExpr )
 		s += ",";
@@ -3508,7 +3527,6 @@ Element* CegoSelect::toElement() const
 
     pSelectElement->addContent(pSelectionElement);
     
-
     Element *pCOListElement = new Element( XML_COLIST_ELEMENT );
     CegoContentObject** pCO = _coList.First();
     while (pCO)
