@@ -8,22 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -31,7 +16,7 @@
 // 
 // Description: This class implements the SQL select query algorithm for the Cego database system 
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -45,7 +30,6 @@
 // cego includes
 #include "CegoProcBlock.h"
 #include "CegoDefs.h"
-#include "CegoTableDesc.h"
 #include "CegoFactor.h"
 #include "CegoDistManager.h"
 #include "CegoDistCursor.h"
@@ -59,6 +43,7 @@
 #include "CegoTerm.h"
 #include "CegoExpr.h"
 #include "CegoContentObject.h"
+#include "CegoQueryCache.h"
 
 class CegoProcBlock;
 
@@ -66,7 +51,9 @@ class CegoSelect {
     
 public:
 
-    CegoSelect(char* buf, CegoDistManager* pGTM);
+    enum SelectMode { PLAIN, AGGREGATION, GROUPING };
+	
+    CegoSelect(char* buf, CegoDistManager* pGTM, int tabSetId);
     CegoSelect(Element* pSelectElement, CegoDistManager* pGTM);
     CegoSelect(ListT<CegoContentObject*>& coList,
 	       ListT<CegoExpr*>& exprList, 
@@ -93,17 +80,13 @@ public:
 
     bool nextTuple(ListT<CegoField>& jfl);
 
-    void enableCache();
-    void disableCache();
-
-    bool isCacheEnabled();
-
-    void setCaching(bool doCaching);
-    bool isCaching();
-    bool isCached();
-
+    bool isCached() const;
+    Chain getQueryId() const;
+    
+    void getObjectList( SetT<CegoObject>& objList ) const;
+    
     void checkValidRef();
-    void setParentJoinBuf(ListT<CegoField>* pParentJoinBuf = 0 , int size = 0);
+    void setParentJoinBuf(ListT<CegoField>** pParentJoinBuf = 0);
     void setProcBlock(CegoProcBlock *pBlock);
 
     int evalExtTableReferences(CegoContentObject* pCO, const ListT<CegoField>& fl);
@@ -111,7 +94,8 @@ public:
     CegoPredDesc* getPredicate();
     void getSchema(ListT<CegoField>& schema);
     Chain toChain(const Chain& indent = Chain("")) const;
-
+    Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    
     Element* toElement() const;
     void fromElement(Element *pSelectElement, CegoDistManager* pGTM);
     
@@ -119,7 +103,7 @@ public:
     ListT<CegoField> getFieldList() const;
 
     void encode(char *buf);
-    void decode(char *buf, CegoDistManager* pGTM);
+    void decode(char *buf, CegoDistManager* pGTM, int tabSetId);
     int getEncodingLength() const;
     
     void setViewCond(const CegoAttrCond& attrCond);
@@ -137,8 +121,6 @@ private:
     bool isPrepared() const;
     void analyzeJoin();
 
-    enum SelectMode { PLAIN, AGGREGATION, GROUPING };
-
     bool nextAggTuple(ListT<CegoField>& jfl);
     bool nextOrderedTuple(ListT<CegoField>& jfl);
     bool nextResultTuple(ListT<CegoField>& jfl, ListT<CegoField>& ofl);
@@ -158,7 +140,10 @@ private:
     void filterCondRef(CegoCondDesc* pC, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
     void filterAttrRef(CegoAttrDesc* pAttrDesc, ListT<CegoField>& inSchema, ListT<CegoField>& outSchema);
 
-    void evalSelection(ListT<CegoField>& ol);
+    // the evalSelection finalizes the selection procedure by evaluating the expressions defined in 
+    // the selection part. Either these are plain attributes, expressions or aggregation functions 
+    // the result is given in the outList parameter
+    void evalSelection(ListT<CegoExpr*>& exprList, ListT<CegoField>** joinBuf, bool isAgg, ListT<CegoField>& ol);
 
     void buildJoinTables();
     void buildJoinRefs();
@@ -175,6 +160,8 @@ private:
     bool matchFieldValue(const CegoFieldValue& fv1, const CegoFieldValue& fv2, CegoComparison comp);    
     void setAggregationValue(CegoExpr *pExpr, ListT<CegoField>& jfl);
 
+    void addObject4CO(CegoContentObject *pCO, SetT<CegoObject>& objList) const;
+    
     ListT<CegoPredDesc*> _viewConjunctionList;
     ListT<CegoPredDesc*> _conjunctionList;
 
@@ -197,9 +184,8 @@ private:
 
     int _dcount;
 
-    ListT<CegoField> _joinBuf[TABMNG_MAXJOINLEVEL];
-    ListT<CegoField>*  _pParentJoinBuf;
-    int _parentJoinBufSize;
+    ListT<CegoField>* _joinBuf[TABMNG_MAXJOINLEVEL];
+    ListT<CegoField>**  _pParentJoinBuf;
 
     CegoDistCursor*_pTC[TABMNG_MAXJOINLEVEL];
 
@@ -210,7 +196,7 @@ private:
 
     bool _firstTuple[TABMNG_MAXJOINLEVEL];
 
-    Logger* _pLogger;
+    CegoDatabaseManager *_pDBMng;
 
     int _aggregationCount;
     int _nextAid;
@@ -234,8 +220,16 @@ private:
     ListT<CegoField> _orderedSchema;
     ListT<CegoField> _dfl;
 
-    ListT< ListT<CegoField> > _queryCache;
-
+    bool _cacheEnabled;
+    CegoQueryCache *_pCache;
+    ListT<CegoField> _cacheSchema;
+    CegoFieldValue*** _pCacheArray;
+    ListT< ListT<CegoFieldValue> >* _pCacheList;
+    int _cacheRows;
+    unsigned long _cacheEntrySize;
+    bool _isCached;
+    Chain _queryId;
+        
     CegoOrderCursor *_pOrderCursor;
     CegoOrderSpace* _pOrderSpace;
 
@@ -253,11 +247,6 @@ private:
 
     int _extRefCount;
 
-    bool _doCaching;
-    bool _isCached;
-    bool _firstFromCache;
-    bool _cacheEnabled;
-
     unsigned long _modId;
 
 };
